import sys
from limix.core.type.cached import cached
from limix.core.utils import assert_make_float_array
from limix.core.utils import assert_finite_array
import scipy as sp
import numpy as np
from covar_base import Covariance
import scipy.spatial as SS

class SQExpCov(Covariance):
    """
    Unidimensional squared exponential covariance function (kernel) for GP regression.
    A unidimensional squared exponential covariance function has two parameters:
        scale:      scale of the covariance (propto explained variance)
        length:     length scale of the input dimension
    """
    def __init__(self, X, Xstar=None):
        """
        X:          [dim, 1] input matrix
        Xstar:      [dim_star, 1] out-of-sample input matrix
        """
        Covariance.__init__(self)
        self._scale_act = True
        self._length_act = True

        X = assert_make_float_array(X, "X")
        assert_finite_array(X)
        self.X = X

        if Xstar is not None:
            Xstar = assert_make_float_array(Xstar, "Xstar")
            assert_finite_array(Xstar)

        self.Xstar = Xstar
        self.params = np.zeros(2)

    def get_input_dim(self):
        return self.X.shape[1]


    #####################
    # Properties
    #####################
    @property
    def scale(self):
        return sp.exp(self.params[0])

    @property
    def length(self):
        return sp.exp(self.params[1])

    @property
    def scale_ste(self):
        if not self._scale_act:
            raise ValueError("Scale is an inactive.")
        if self.getFIinv() is None:
            R = None
        else:
            i = self._index2actindex(0)
            R = sp.sqrt(self.getFIinv()[i,i])
        return R

    @property
    def length_ste(self):
        if not self._length_act:
            raise ValueError("Length is an inactive.")
        if self.getFIinv() is None:
            R = None
        else:
            i = self._index2actindex(1)
            R = sp.sqrt(self.getFIinv()[i,i])
        return R

    @property
    def X(self):
        return self._X

    @property
    def Xstar(self):
        return self._Xstar

    #####################
    # Setters
    #####################
    @scale.setter
    def scale(self,value):
        assert value>=0, 'Scale must be >=0'
        self.params[0] = sp.log(value)
        self.clear_all()

    @length.setter
    def length(self,value):
        assert value>=0, 'Length must be >=0'
        self.params[1] = sp.log(value)
        self.clear_all()

    @X.setter
    def X(self,value):
        self._X = value
        self.initialize(value.shape[0])
        self.clear_cache('X')
        self.clear_all()

    @Xstar.setter
    def Xstar(self,value):
        if value is None:
            self._use_to_predict = False
        else:
            assert value.shape[1]==self.X.shape[1], 'Dimension mismatch'
            self._use_to_predict = True
        self._Xstar = value
        self.clear_cache('Xstar')
        self._notify()

    @Covariance.use_to_predict.setter
    def use_to_predict(self,value):
        assert self.Xstar is not None, 'set Xstar!'
        self._use_to_predict = value
        self._notify()

    #####################
    # Activation handling
    #####################
    @property
    def act_scale(self):
        return self._scale_act

    @act_scale.setter
    def act_scale(self, act):
        self._scale_act = bool(act)
        self._notify()

    @property
    def act_length(self):
        return self._length_act

    @act_length.setter
    def act_length(self, act):
        self._length_act = bool(act)
        self._notify()

    def _actindex2index(self, i):
        return i + int(not self._scale_act)

    def _index2actindex(self, i):
        return i - int(not self._scale_act)

    #####################
    # Params handling
    #####################
    def setParams(self, params):
        sel = np.asarray((self._scale_act, self._length_act))
        if np.sum(sel) != len(params):
            raise ValueError("The number of parameters passed to setParams "
                             "differs from the number of active parameters.")
        self.params[sel] = params
        self.clear_all()

    def getParams(self):
        sel = np.asarray((self._scale_act, self._length_act))
        return self.params[sel]

    def getNumberParams(self):
        return np.sum([self._scale_act, self._length_act])

    #####################
    # Cached
    #####################
    @cached('X')
    def E(self):
        rv = SS.distance.pdist(self.X,'euclidean')**2
        rv = SS.distance.squareform(rv)
        return rv

    @cached(['X', 'Xstar', 'covar_base'])
    def Kcross(self):
        assert self.Xstar.shape[1]==1, 'only implemented for 1-dim input'
        Estar = (self.Xstar - self.X.T)**2
        return  self.scale * sp.exp(-Estar/(2*self.length))

    @cached('covar_base')
    def K(self):
        return self.scale * sp.exp(-self.E()/(2*self.length))

    @cached('covar_base')
    def K_grad_i(self, i):
        if i >= int(self._scale_act) + int(self._length_act):
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")

        r = self.K_grad_interParam_i(i)
        i = self._actindex2index(i)
        if i==0:
            r *= self.scale
        elif i==1:
            r *= self.length
        else:
            assert False, 'There is no index %d on sqexp.' %i
        return r

    @cached('covar_base')
    def K_hess_i_j(self, i, j):
        if i >= int(self._scale_act) + int(self._length_act) or j >= int(self._scale_act) + int(self._length_act):
            raise ValueError("Trying to retrieve the hessian over a "
                             "parameter that is inactive.")

        i = self._actindex2index(i)
        j = self._actindex2index(j)
        if i==0 and j==0:
            r = self.K()
        elif (i==0 and j==1) or (i==1 and j==0):
            r = self.K_grad_i(1)
        elif i==1 and j==1:
            r = self.K_grad_i(1) - self.K_grad_i(0)
            r*= self.E() / (2*self.length)
        else:
            assert False, 'There is no index %d on sqexp.' %i
        return r

    ####################
    # Interpretable Params
    ####################
    def getInterParams(self):
        params = []
        if self._scale_act:
            params.append(self.scale)
        if self._length_act:
            params.append(self.length)
        return np.array(params)

    def K_grad_interParam_i(self, i):
        i = self._actindex2index(i)
        if i == 0:
            r = sp.exp(-self.E()/(2*self.length))
        elif i == 1:
            A = sp.exp(-self.E()/(2*self.length))*self.E()
            r = self.scale * A / (2*self.length**2)
        else:
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")
        return r



"""Matrix linear algebra routines needed for GP models"""



import scipy as SP
import scipy.linalg as linalg
import logging



def solve_chol(A,B):
    """
    Solve cholesky decomposition::
    
        return A\(A'\B)

    """
    # X = linalg.solve(A,linalg.solve(A.transpose(),B))
    # much faster version
    X = linalg.cho_solve((A, True), B)
    return X






def jitChol(A, maxTries=10, warning=True):

    """Do a Cholesky decomposition with jitter.
    
    Description:
    
    
    U, jitter = jitChol(A, maxTries, warning) attempts a Cholesky
     decomposition on the given matrix, if matrix isn't positive
     definite the function adds 'jitter' and tries again. Thereafter
     the amount of jitter is multiplied by 10 each time it is added
     again. This is continued for a maximum of 10 times.  The amount of
     jitter added is returned.
     Returns:
      U - the Cholesky decomposition for the matrix.
      jitter - the amount of jitter that was added to the matrix.
     Arguments:
      A - the matrix for which the Cholesky decomposition is required.
      maxTries - the maximum number of times that jitter is added before
       giving up (default 10).
      warning - whether to give a warning for adding jitter (default is True)

    See also
    CHOL, PDINV, LOGDET


    Copyright (c) 2005, 2006 Neil D. Lawrence
    
    """
    jitter = 0
    i = 0

    while(True):
        try:
            # Try --- need to check A is positive definite
            if jitter == 0:
                jitter = abs(SP.trace(A))/A.shape[0]*1e-6
                LC = linalg.cholesky(A, lower=True)
                return LC.T, 0.0
            else:
                if warning:
                    # pdb.set_trace()
		    # plt.figure()
		    # plt.imshow(A, interpolation="nearest")
		    # plt.colorbar()
		    # plt.show()
                    logging.error("Adding jitter of %f in jitChol()." % jitter)
                LC = linalg.cholesky(A+jitter*SP.eye(A.shape[0]), lower=True)

                return LC.T, jitter
        except linalg.LinAlgError:
            # Seems to have been non-positive definite.
            if i<maxTries:
                jitter = jitter*10
            else:
                raise linalg.LinAlgError, "Matrix non positive definite, jitter of " +  str(jitter) + " added but failed after " + str(i) + " trials."
        i += 1
    return LC


def jitEigh(A,maxTries=10,warning=True):
    """
    Do a Eigenvalue Decompsition with Jitter,

    works as jitChol
    """
    warning = True
    jitter = 0
    i = 0

    while(True):
        if jitter == 0:
            jitter = abs(SP.trace(A))/A.shape[0]*1e-6
            S,U = linalg.eigh(A)

        else:
            if warning:
                # pdb.set_trace()
		# plt.figure()
		# plt.imshow(A, interpolation="nearest")
		# plt.colorbar()
		# plt.show()
                logging.error("Adding jitter of %f in jitEigh()." % jitter)
            S,U = linalg.eigh(A+jitter*SP.eye(A.shape[0]))

        if S.min()>1E-10:
            return S,U
            
        if i<maxTries:
            jitter = jitter*10
        i += 1
            
    raise linalg.LinAlgError, "Matrix non positive definite, jitter of " +  str(jitter) + " added but failed after " + str(i) + " trials."
      



import unittest

from limix.core.mean.mean_base import MeanBase as lin_mean
from limix.core.covar.sqexp import SQExpCov
from limix.core.covar.fixed import FixedCov
from limix.core.covar.combinators import SumCov
from limix.core.gp.gp_base import GP
from limix.utils.check_grad import mcheck_grad

import numpy as np
import scipy as sp


class TestGPBase(unittest.TestCase):
    def setUp(self):
        np.random.seed(1)

        # generate data
        N = 400
        s_x = 0.05
        s_y = 0.1
        X = (sp.linspace(0,2,N)+s_x*sp.randn(N))[:,sp.newaxis]
        Y = sp.sin(X)+s_y*sp.randn(N,1)
        Y-= Y.mean(0)
        Y/= Y.std(0)

        Xstar = sp.linspace(0,2,1000)[:,sp.newaxis]

        # define mean term
        F = 1.*(sp.rand(N,2)<0.2)
        mean = lin_mean(Y,F)

        # define covariance matrices
        covar1 = SQExpCov(X,Xstar=Xstar)
        covar2 = FixedCov(sp.eye(N))
        covar  = SumCov(covar1,covar2)

        # define gp
        self._gp = GP(covar=covar,mean=mean)

    def test_grad(self):

        gp = self._gp

        def func(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            return gp.LML()

        def grad(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            grad = gp.LML_grad()
            return grad['covar'][i]

        x0 = gp.getParams()['covar']
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=5)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# Copyright(c) 2014, The mtSet developers (Francesco Paolo Casale, Barbara Rakitsch, Oliver Stegle)
# All rights reserved.

from limix.mtSet.core.preprocessCore import preprocess
from optparse import OptionParser

def entry_point():
    parser = OptionParser()
    parser.add_option("--bfile", dest='bfile', type=str, default=None)
    parser.add_option("--cfile", dest='cfile', type=str, default=None)
    parser.add_option("--pfile", dest='pfile', type=str, default=None)
    parser.add_option("--nfile", dest='nfile', type=str, default=None)
    parser.add_option("--wfile", dest='wfile', type=str, default=None)
    parser.add_option("--ffile", dest='ffile', type=str, default=None)
    parser.add_option("--trait_idx",dest='trait_idx',type=str, default=None)

    parser.add_option("--compute_covariance", action="store_true",dest="compute_cov",default=False)
    parser.add_option("--compute_PCs", dest="compute_PCs",default=0,type=int)


    parser.add_option("--plink_path", dest='plink_path', type=str, default='plink')
    parser.add_option("--sim_type",   dest='sim_type', type=str, default='RRM')

    parser.add_option("--fit_null", action="store_true",dest="fit_null",default=False)

    parser.add_option("--precompute_windows", action="store_true", dest='precompute_windows', default=False)
    parser.add_option("--window_size", dest='window_size', type=int, default=3e4)
    parser.add_option("--plot_windows", action='store_true', dest='plot_windows',default=False)

    (options, args) = parser.parse_args()

    preprocess(options)



"""
Created on Sep 24, 2013
@author: casale
"""
import scipy as SP
import scipy.linalg as LA
import pdb

import sys
sys.path.append('./../../..')
import plink_reader


def genBinormal(dim1,dim2,percSign=0.5,std=1e-1):
    rv = (2*(SP.rand(dim1,dim2)>percSign)-1)+std*SP.randn(dim1,dim2)
    return rv

def selectRnd(n_sel,n_all):
    rv = SP.zeros(n_all)
    rv[:n_sel] = 1
    SP.random.shuffle(rv)
    rv = rv==1
    return rv

class CSimulator:
    """
    this class takes care of phenotype generation in a flexible way
    """

    def __init__(self,bfile=None,XX=None,X=None,chrom=None,pos=None,P=1,trait_effect='shared'):
        """
        X: genotype matrix
        traitNum: number of traits to be considered
        """
        self.bfile = bfile
        self.N = XX.shape[0]
        self.P = P
        self.XX = XX

        self.X = X
        self.chrom = chrom
        self.pos   = pos
        
        self.trait_effect = trait_effect

    def setTraitEffect(self,trait_effect):
        self.trait_effect = trait_effect
    
    def getRegion(self,size=3e4,min_nSNPs=1,chrom_i=None,pos_min=None,pos_max=None):
        """
        Sample a region from the piece of genotype X, chrom, pos
        minSNPnum:  minimum number of SNPs contained in the region
        Ichrom:  restrict X to chromosome Ichrom before taking the region
        cis:        bool vector that marks the sorted region
        region:  vector that contains chrom and init and final position of the region
        """
        if (self.chrom is None) or (self.pos is None):
            bim = plink_reader.readBIM(self.bfile,usecols=(0,1,2,3))
            chrom = SP.array(bim[:,0],dtype=int)
            pos   = SP.array(bim[:,3],dtype=int)
        else:
            chrom = self.chrom
            pos   = self.pos
            
        if chrom_i is None:
            n_chroms = chrom.max()
            chrom_i  = int(SP.ceil(SP.rand()*n_chroms))

        pos   = pos[chrom==chrom_i]
        chrom = chrom[chrom==chrom_i]

        ipos = SP.ones(len(pos),dtype=bool)
        if pos_min is not None:
            ipos = SP.logical_and(ipos,pos_min<pos)

        if pos_max is not None:
            ipos = SP.logical_and(ipos,pos<pos_max)

        pos = pos[ipos]
        chrom = chrom[ipos]
        
        if size==1:
            # select single SNP
            idx = int(SP.ceil(pos.shape[0]*SP.rand()))
            cis  = SP.arange(pos.shape[0])==idx
            region = SP.array([chrom_i,pos[idx],pos[idx]])
        else:
            while 1:
                idx = int(SP.floor(pos.shape[0]*SP.rand()))
                posT1 = pos[idx]
                posT2 = pos[idx]+size
                if posT2<=pos.max():
                    cis = chrom==chrom_i
                    cis*= (pos>posT1)*(pos<posT2)
                    if cis.sum()>min_nSNPs: break
            region = SP.array([chrom_i,posT1,posT2])

        start = SP.nonzero(cis)[0].min()
        nSNPs  = cis.sum()

        if self.X is None:
            rv = plink_reader.readBED(self.bfile,useMAFencoding=True,start = start, nSNPs = nSNPs,bim=bim)
            Xr = rv['snps']
        else:
            Xr = self.X[:,start:start+nSnps]
            
        return Xr, region


    def genRegionTerm(self,X,vTot=0.1,pCausal=0.10,nCausal=None,pCommon=1.,nCommon=None,plot=False,distribution='biNormal'):
        """
        Generate population structure term
        Population structure is simulated by background SNPs
        
        beta_pdf:        pdf used to generate the regression weights
                          for now either Normal or fixed
        variance:        variance of the term
        percCausal:    percentage of causal SNPs
        Xcausal:          set of SNPs being causal
        """
        S = X.shape[1]

        # number of causal, common, specific
        if nCausal==None:
            nCausal=int(SP.floor(pCausal*S))
        if nCommon==None:
            nCommon = round(pCommon*nCausal)
        nSpecific = self.P*(nCausal-nCommon)

        # common SNPs
        if nCommon>0:
            if distribution=='biNormal':
                Bc  = SP.dot(genBinormal(nCommon,self.P),self.genTraitEffect(distribution=distribution))
            elif distribution=='normal':
                Bc  = SP.dot(self.genWeights(nCommon,self.P),self.genTraitEffect(distribution=distribution))
            Ic  = selectRnd(nCommon,S)
            Yc  = SP.dot(X[:,Ic],Bc)
            Yc *= SP.sqrt(nCommon/Yc.var(0).mean())
        else:
        
            Yc = SP.zeros((self.N,self.P))

        # indipendent signal
        if nSpecific>0:
            Is  = selectRnd(nSpecific,S*self.P).reshape(S,self.P)
            if distribution=='biNormal':
                Bi  = Is*genBinormal(S,self.P)
            elif distribution=='normal':
                Bi  = Is*SP.randn(S,self.P)
            Yi  = SP.dot(X,Bi)
            Yi *= SP.sqrt(nSpecific/(Yi.var(0).mean()*self.P))
        else:
            Yi = SP.zeros((self.N,self.P))
        
        
        Y   = Yc+Yi 
        Yc *= SP.sqrt(vTot/Y.var(0).mean())
        Yi *= SP.sqrt(vTot/Y.var(0).mean())

        if plot:
            import pylab as PL
            PL.ion()
            for p in range(self.P):
                PL.subplot(self.P,1,p+1)
                PL.plot(SP.arange(S)[Ic],Bc[:,p],'o',color='y')
                _Is = Is[:,p]
                if _Is.sum()>0:
                    PL.plot(SP.arange(S)[_Is],Bi[_Is,p],'o',color='r') 
                #PL.ylim(-2,2)
                PL.plot([0,S],[0,0],'k')

        return Yc, Yi


    def _genBgTerm_fromSNPs(self,vTot=0.5,vCommon=0.1,pCausal=0.5,plot=False):
        """ generate  """

        if self.X is None:
            print 'Reading in all SNPs. This is slow.'
            rv = plink_reader.readBED(self.bfile,useMAFencoding=True)
            X  = rv['snps']
        else:
            X  = self.X
        
        S  = X.shape[1]
        vSpecific = vTot-vCommon

        # select causal SNPs
        nCausal = int(SP.floor(pCausal*S))
        Ic = selectRnd(nCausal,S)
        X = X[:,Ic]

        # common effect
        Bc  = SP.dot(self.genWeights(nCausal,self.P),self.genTraitEffect())        
        Yc  = SP.dot(X,Bc)
        Yc *= SP.sqrt(vCommon/Yc.var(0).mean())

        # indipendent effect
        Bi  = SP.randn(nCausal,self.P)
        Yi  = SP.dot(X,Bi)
        Yi *= SP.sqrt(vSpecific/Yi.var(0).mean())

        if plot:
            import pylab as PL
            PL.ion()
            for p in range(self.P):
                PL.subplot(self.P,1,p+1)
                PL.plot(SP.arange(self.X.shape[1])[Ic],Bc[:,p],'o',color='y',alpha=0.05)
                PL.plot(SP.arange(self.X.shape[1])[Ic],Bi[:,p],'o',color='r',alpha=0.05) 
                #PL.ylim(-2,2)
                PL.plot([0,Ic.shape[0]],[0,0],'k')

        return Yc, Yi

    def _genBgTerm_fromXX(self,vTot,vCommon,XX,a=None,c=None):
        """
        generate background term from SNPs

        Args:
            vTot: variance of Yc+Yi
            vCommon: variance of Yc
            XX: kinship matrix
            a: common scales, it can be set for debugging purposes
            c: indipendent scales, it can be set for debugging purposes
        """
        vSpecific = vTot-vCommon

        SP.random.seed(0)
        if c==None: c = SP.randn(self.P)
        XX += 1e-3 * SP.eye(XX.shape[0])
        L = LA.cholesky(XX,lower=True)
        
        # common effect
        R = self.genWeights(self.N,self.P)
        A = self.genTraitEffect()
        if a is not None: A[0,:] = a
        Yc = SP.dot(L,SP.dot(R,A))
        Yc*= SP.sqrt(vCommon)/SP.sqrt(Yc.var(0).mean())

        # specific effect
        R = SP.randn(self.N,self.P)
        Yi = SP.dot(L,SP.dot(R,SP.diag(c)))
        Yi*= SP.sqrt(vSpecific)/SP.sqrt(Yi.var(0).mean())

        return Yc, Yi
        

    def genBgTerm(self,vTot=0.5,vCommon=0.1,pCausal=0.5,XX=None,use_XX=False,a=None,c=None,plot=False):
        """ generate  """
        if use_XX:
            if XX is None:  XX = self.XX
            assert XX is not None, 'Simulator: set XX!'
            Yc,Yi = self._genBgTerm_fromXX(vTot,vCommon,XX,a=a,c=c)
        else:
            Yc,Yi = self._genBgTerm_fromSNPs(vTot=vTot,vCommon=vCommon,pCausal=pCausal,plot=plot)
        return Yc, Yi

    def genHidden(self,nHidden=10,vTot=0.5,vCommon=0.1):
        """ generate  """

        vSpecific = vTot-vCommon

        # generate hidden
        X = self.genWeights(self.N,nHidden)

        # common effect
        H = self.genWeights(nHidden,self.P)
        Bc = SP.dot(H,self.genTraitEffect())
        Yc  = SP.dot(X,Bc)
        Yc *= SP.sqrt(vCommon/Yc.var(0).mean())
        
        # indipendent effect
        Bi  = SP.randn(nHidden,self.P)
        Yi  = SP.dot(X,Bi)
        Yi *= SP.sqrt(vSpecific/Yi.var(0).mean())

        return Yc,Yi

    def genTraitEffect(self,distribution='normal'):

        W = SP.zeros((self.P,self.P))
        if self.trait_effect=='shared':
            

            if distribution=='normal':
                W[0,:] = SP.random.randn(1,self.P)
            else:
                W[0,:] = genBinormal(1,self.P)

        elif self.trait_effect=='tanh':
            assert distribution=='normal', 'tanh trait effect is only implemented for normal distributed effects'
            X = 10*SP.linspace(0,1,self.P)-5
            a = - SP.absolute(SP.random.randn())
            c = - SP.absolute(SP.random.randn())
            Y = SP.tanh(a*X + c)
            W[0,:] = Y

        return W

    def genWeights(self,N,P):
        if self.trait_effect=='shared':
            return SP.random.randn(N,P)

        elif self.trait_effect=='tanh':
            # ensure that weights are positive...
            return SP.absolute(SP.random.randn(N,P))
            
    def genNoise(self,vTot=0.4,vCommon=0.2):

        vSpecifc = vTot-vCommon

        # common
        Yc  = SP.dot(self.genWeights(self.N,self.P),self.genTraitEffect())
        Yc *= SP.sqrt(vCommon/Yc.var(0).mean())
        
        # independent
        Yi  = SP.randn(self.N,self.P)
        Yi *= SP.sqrt(vSpecifc/Yi.var(0).mean())

        return Yc,Yi


    def genPheno(self,Xr,
        vTotR=0.1,nCommonR=5,nCausalR=10,distribution='biNormal',
        vCommonBg=0.1,vTotBg=0.4,pCausalBg=0.5,XX=None,use_XX=False,
        vCommonH=0.1,vTotH=0.2,nHidden=10,
        vCommonN=0.,vTotN=0.3,standardize=True):

      
        YRc,YRi = self.genRegionTerm(Xr,vTot=vTotR,nCommon=nCommonR,nCausal=nCausalR,distribution=distribution)
        YGc,YGi = self.genBgTerm(vCommon=vCommonBg,vTot=vTotBg,pCausal=pCausalBg,XX=XX,use_XX=use_XX)
        YHc,YHi = self.genHidden(vCommon=vCommonH,vTot=vTotH,nHidden=nHidden)
        YNc,YNi = self.genNoise(vCommon=vCommonN,vTot=vTotN)        
        Y = YRc+YRi+YGc+YGi+YHc+YHi+YNc+YNi

        if standardize:
            Y -= Y.mean(0)
            Y /= Y.std(0)

        info = {'YRc':YRc,'YRi':YRi,
                'YGc':YGc,'YGi':YGi,
                'YHc':YHc,'YHi':YHi,
                'YNc':YNc,'YNi':YNi}

        return Y, info



if __name__ == "__main__":
    import matplotlib.pylab as PLT
    
    N = 10
    F = 1000
    K = 10
    
    X = SP.random.rand(N,F)
    X[X<0.5] = 0
    X[X>0.5] = 1
    chrom = SP.ones(F)
    pos   = SP.arange(F)
    P     = 20

    Z = SP.random.randn(N,K)
    XX = SP.dot(Z,Z.T)
    XX/= SP.diag(XX).mean()
    XX = SP.sqrt(0.7)*XX + SP.sqrt(0.3)*SP.eye(N)
    
    model = CSimulator(XX=XX,X=X,chrom=chrom,pos=pos,P=20,trait_effect='shared')
    model.setTraitEffect('tanh')
  
    Y,info = model.genPheno(X[:,[0]],
        vTotR=0.1,nCommonR=1,nCausalR=1,distribution='normal',
        vCommonBg=0.2,vTotBg=0.3,pCausalBg=0.5,XX=XX,use_XX=True,
        vCommonH=0.3,vTotH=0.4,nHidden=10,
        vCommonN=0.0,vTotN=0.2,standardize=False)
    
    

    fig = PLT.figure()
    fig.add_subplot(321)
    PLT.title('Total')
    PLT.plot(Y.T,'x')

    fig.add_subplot(322)
    PLT.title('Hidden')
    PLT.plot(info['YHc'].T)

    fig.add_subplot(323)
    PLT.title('Genetic')
    PLT.plot(info['YGc'].T)

    fig.add_subplot(324)
    PLT.title('Region')
    PLT.plot(info['YRc'].T)

    fig.add_subplot(325)
    PLT.title('Common')
    PLT.plot(info['YRc'].T+info['YHc'].T+info['YGc'].T+info['YNc'].T)
    
    pdb.set_trace()



import numpy as np
import scipy as sp
import scipy.linalg as LA
from covar_base import Covariance
from freeform import FreeFormCov
from limix.core.type.cached import cached
import pdb

import logging as LG

class DirIndirCov(Covariance):
    """
    Covariance matrix for decomposing direct and social genetic effects
    """
    def __init__(self, kinship, design, kinship_cm = None, kinship_cross = None, jitter = 1e-4):
        if kinship_cm is None:      kinship_cm = kinship
        if kinship_cross is None:   kinship_cross = kinship
        ff_dim = 2
        self.covff = FreeFormCov(ff_dim, jitter = 1e-4)
        self._K = kinship
        self._ZK = sp.dot(design, kinship_cross.T)
        self._KZ = sp.dot(kinship_cross, design.T)
        self._ZKZ = sp.dot(design, sp.dot(kinship_cm, design.T))
        Covariance.__init__(self, kinship.shape[0])

    def dirIndirCov_K(self):
        return self.covff.K()

    def dirIndirCov_K_ste(self):
        return self.covff.K_ste()

    #####################
    # Properties
    #####################
    @property
    def variance(self):
        return self.covff.variance

    @property
    def correlation(self):
        return self.covff.correlation

    #####################
    # Params handling
    #####################
    def getParams(self):
        return self.covff.getParams()

    def setParams(self,params):
        self.covff.setParams(params)
        self.clear_all()

    def getNumberParams(self):
        return self.covff.getNumberParams()

    def setCovariance(self,cov):
        """ set hyperparameters from given covariance """
        return self.covff.setCovariance(cov)

    #####################
    # Cached
    #####################
    @cached('covar_base')
    def K(self):
        C = self.covff.K()
        RV  = C[0,0] * self._K
        RV += C[0,1] * (self._KZ + self._ZK)
        RV += C[1,1] * self._ZKZ
        return RV

    @cached('covar_base')
    def K_grad_i(self,i):
        Cgrad = self.covff.K_grad_i(i)
        RV  = Cgrad[0,0] * self._K
        RV += Cgrad[0,1] * (self._KZ + self._ZK)
        RV += Cgrad[1,1] * self._ZKZ
        return RV

    ####################
    # Interpretable Params
    ####################
    def getInterParams(self):
        return self.covff.getInterParams()

    def K_grad_interParam_i(self,i):
        Cgrad = self.covff.K_grad_interParam_i(i)
        RV  = Cgrad[0,0] * self._K
        RV += Cgrad[0,1] * (self._KZ + self._ZK)
        RV += Cgrad[1,1] * self._ZKZ
        return RV

    def setFIinv(self, value):
        self.covff.setFIinv(value)

    def getFIinv(self):
        return self.covff.getFIinv()

if __name__ == '__main__':
    # generate data
    n = 100
    f = 10
    X  = 1.*(sp.rand(n,f)<0.2)
    X -= X.mean(0); X /= X.std(0)
    kinship  = sp.dot(X,X.T)
    kinship /= kinship.diagonal().mean()
    design = sp.zeros((n,n))
    for i in range(n/2):
        design[2*i,2*i+1] = 1
        design[2*i+1,2*i] = 1

    # test covariance
    cov = DirIndirCov(kinship,design)
    cov.setRandomParams()
    print cov.K()
    print cov.K_grad_i(0)



import numpy as np
import scipy.linalg as la
import scipy.stats as stats
from limix.core.cobj import *
from limix.core.mean.mean import compute_X1KX2, compute_XYA
import limix.utils.psd_solve as psd_solve

class LmmKronecker(cObject):
    pass

    def __init__(self, gp=None):
        '''
        Input:
        forcefullrank   : if True, then the code always computes K and runs cubically
					        (False)
        '''
        self._gp = gp
        
        gp.set_reml(False)#currently only ML is supported
        self._LL_0 = self._gp.LML()
        #self.clear_cache('LL_snps','test_snps')
    
    def addFixedEffect(self,F,A=None):
        self._gp.mean.addFixedEffect(F=F, A=A)
        self._LL_0 = self._gp.LML()
        #self.clear_cache('LL_snps','test_snps')
        pass

    def LL_snps(self, snps, Asnps=None, inter=None, identity_trick=False):
        """
        compute log likelihood for SNPs
        """
        LL_snps = np.zeros(snps.shape[1])
        LL_new = np.zeros(snps.shape[1])
        LL_snps_0 = np.zeros(snps.shape[1])
        #index_snpterm = self.gp.mean.n_terms
        if inter is None:

            LL_snps_0[:] = self._LL_0
            for i_snp in xrange(snps.shape[1]):
                #correlated but not identical:
                if 0:
                    self._gp.mean.clear_cache('Fstar','Astar','Xstar','Xhat',
                         'Areml','Areml_eigh','Areml_chol','Areml_inv','beta_hat','B_hat',
                         'LRLdiag_Xhat_tens','Areml_grad',
                         'beta_grad','Xstar_beta_grad','Zstar','DLZ')
                    Areml_small1 = self._gp.mean.Areml()
                LL_new[i_snp],beta,Areml1,Areml_11 = self.LML_blockwise(snp=np.dot(self._gp.mean.Lr,snps[:,i_snp:i_snp+1]), Asnp=Asnps, identity_trick=True)
                #worse:
                #LL_new[i_snp],beta = self.LML_blockwise(snp=snps[:,i_snp:i_snp+1], Asnp=Asnps, identity_trick=True)
                if 0:
                    Areml_small2 = self._gp.mean.Areml()
                    diff_small = Areml_small1-Areml_small2
                    absdiff_small=np.absolute(diff_small)
                    print np.max(absdiff_small)
                self._gp.mean.addFixedEffect(F=snps[:,i_snp:i_snp+1], A=Asnps)
                
                if 0:
                    self._gp.mean.clear_cache('Fstar','Astar','Xstar','Xhat',
                         'Areml','Areml_eigh','Areml_chol','Areml_inv','beta_hat','B_hat',
                         'LRLdiag_Xhat_tens','Areml_grad',
                         'beta_grad','Xstar_beta_grad','Zstar','DLZ')
                if 0:
                    Areml2_prior = self._gp.mean.Areml()
                LL__ = self._gp.LML(identity_trick=identity_trick)
                
                Areml2 = self._gp.mean.Areml()
                diff=Areml1-Areml2
                if 0:
                    diff_prior=Areml2 - Areml2_prior
                    absdiff_prior=np.absolute(diff_prior)
                    print np.max(absdiff_prior)
                absdiff=np.absolute(diff)
                print np.max(absdiff)
                print np.max(absdiff[0:-1,0:-1])
                if 0:
                    diff_large_small = Areml1[0:-1,0:-1]-Areml_small2
                    absdiff_large_small=np.absolute(diff_large_small)
                    print np.max(absdiff_large_small)
                    diff_large_small = Areml2[0:-1,0:-1]-Areml_11
                    absdiff_large_small=np.absolute(diff_large_small)
                    print np.max(absdiff_large_small)
                #import ipdb;ipdb.set_trace()
                LL_snps[i_snp] = LL__
                self._gp.mean.removeFixedEffect()
        else:
            for i_snp in xrange(snps.shape[1]):
                self._gp.mean.addFixedEffect(F=snps[:,i_snp:i_snp+1], A=Asnps)
                LL_snps_0[i_snp] = self._gp.LML(identity_trick=identity_trick)
                self._gp.mean.addFixedEffect(F=snps[:,i_snp:i_snp+1]*inter, A=Asnps)
                LL_snps[i_snp] = self._gp.LML(identity_trick=identity_trick)
                self._gp.mean.removeFixedEffect()
                self._gp.mean.removeFixedEffect()
        return LL_snps,LL_snps_0,LL_new


    def LML_blockwise(self, snp, Asnp=None, identity_trick=False, *kw_args):
        """
        calculate LML
        """
        self._gp._update_cache()
        
        if Asnp is None:
            nW_Asnp = self._gp.mean.Ystar().shape[1]	
        else:
            nW_Asnp = Asnp.shape[1]
            	

        #1. const term
        lml  = self._gp.N*self._gp.P*np.log(2.0*np.pi)

        #2. logdet term
        lml += np.sum(np.log(self._gp.cache['Sc2']))*self._gp.N + np.log(self._gp.cache['s']).sum()

        #3. quadratic term
        #quad1 = (self._gp.mean.Zstar(identity_trick=identity_trick)*self._gp.mean.DLZ(identity_trick=identity_trick)).sum()
        
        XKY = self._gp.mean.compute_XKY(M=self._gp.mean.Yhat(), identity_trick=identity_trick)
        beta = self._gp.mean.beta_hat(identity_trick=identity_trick)
        var_total = (self._gp.mean.Yhat()*self._gp.mean.Ystar()).sum()
        var_exp = (XKY*beta).sum()
        
        
        #use blockwise matrix inversion
        #[  Areml,          XcovarXsnp
        #   XcovarXsnp.T    XsnpXsnp    ]
        
        XsnpXsnp = compute_X1KX2(Y=self._gp.mean.Ystar(), D=self._gp.mean.D, X1=snp, X2=snp, A1=Asnp, A2=Asnp)
        XcovarXsnp = np.zeros((self._gp.mean.n_fixed_effs,nW_Asnp*snp.shape[1]))
        n_effs_sum = 0
        for term in xrange(self._gp.mean.n_terms):
            n_effs_term = self._gp.mean.Fstar()[term].shape[1]
            if self._gp.mean.A_identity[term]:
                n_effs_term *= self._gp.P
            else:
                n_effs_term *= self._gp.mean.Astar()[term].shape[1]
            if identity_trick and self._gp.mean.A_identity[term]:
                Astar_term = None
            else:
                Astar_term = self._gp.mean.Astar()[term]
            XcovarXsnp[n_effs_sum:n_effs_term+n_effs_sum,:] = compute_X1KX2(Y=self._gp.mean.Ystar(), D=self._gp.mean.D, X1=self._gp.mean.Fstar()[term], X2=snp, A1=Astar_term, A2=Asnp)
        AXcovarXsnp = self._gp.mean.Areml_solve(XcovarXsnp)
        XsnpXsnp_ = XsnpXsnp - XcovarXsnp.T.dot(AXcovarXsnp)
        #compute beta
       
        #compute a
        snpKY = compute_XYA(DY=self._gp.mean.Yhat(), X=snp, A=Asnp)
        if 0:
            XsnpXsnp_solver = psd_solve.psd_solver(XsnpXsnp_, lower=True, threshold=1e-10,check_finite=True,overwrite_a=False)
            #solve XsnpXsnp \ AXcovarXsnp*beta
            DCbeta = XsnpXsnp_solver.solve(XcovarXsnp.T.dot(beta),overwrite_b=True)
            #solve XsnpXsnp \ a
            Da = XsnpXsnp_solver.solve(snpKY,overwrite_b=False)
            beta_snp = Da-DCbeta
            beta_up = XcovarXsnp.dot(-beta_snp)
            var_expl_snp = (XKY*(beta + beta_up)).sum() + (snpKY*beta_snp).sum()
            beta_all = np.concatenate([beta+beta_up,beta_snp])
        
        

        Areml_11 = self._gp.mean.Areml()
        Areml_12 = XcovarXsnp
        Areml_22 = XsnpXsnp
        Areml1 = np.concatenate((Areml_11,Areml_12),1)
        Areml2 = np.concatenate((Areml_12.T,Areml_22),1)
        Areml_all = np.concatenate((Areml1,Areml2),0)
        Areml_all_solver = psd_solve.psd_solver(Areml_all, lower=True, threshold=1e-10,check_finite=True,overwrite_a=False)
        XKY_all = np.concatenate((XKY,snpKY),0)
        beta_all_ = Areml_all_solver.solve(XKY_all)
        var_expl_all=(XKY_all*beta_all_).sum()

        var_res = var_total - var_expl_all#var_expl_snp
        #import ipdb;ipdb.set_trace()
        lml += var_res
        lml *= 0.5

        #import ipdb;ipdb.set_trace()
        return lml,beta_all_,Areml_all,Areml_11

    def test_snps(self, snps, Asnps=None, inter=None, identity_trick=False):
        """
        test snps for association
        """
        if Asnps is None:
            dof = self._gp.mean.P
        else:
            dof = Asnps.shape[0]
        LL_snps, LL_snps_0,LL_new = self.LL_snps(snps=snps,Asnps=Asnps, inter=inter, identity_trick=identity_trick)
        LRT = 2.0 * (LL_snps_0 - LL_snps)
        LRT_new = 2.0 * (LL_snps_0 - LL_new)
        pv = stats.chi2.sf(LRT,dof)
        pv_new = stats.chi2.sf(LRT_new,dof)
        #import ipdb;ipdb.set_trace()
        return pv,LL_snps,LL_snps_0,LL_new,pv_new






from optparse import OptionParser
import scipy as SP

import os
import simulator as sim
from read_utils import readCovarianceMatrixFile
from read_utils import readBimFile 

def genPhenoCube(sim,Xr,vTotR=4e-3,nCausalR=10,pCommonR=0.8,vTotBg=0.4,pHidd=0.6,pCommon=0.8):
    # region
    nCommonR  = int(SP.around(nCausalR*pCommonR))
    # background
    vCommonBg = pCommon*vTotBg
    # noise
    vTotH   = pHidd*(1-vTotR-vTotBg)
    vTotN   = (1-pHidd)*(1-vTotR-vTotBg)
    vCommonH = pCommon*vTotH
    all_settings = {
        'vTotR':vTotR,'nCommonR':nCommonR,'nCausalR':nCausalR,
        'vTotBg':vTotBg,'vCommonBg':vCommonBg,'pCausalBg':1.,'use_XX':True,
        'vTotH':vTotH,'vCommonH':vCommonH,'nHidden':10,
        'vTotN':vTotN,'vCommonN':0.}

    Y,info = sim.genPheno(Xr,**all_settings)
    return Y,info

def simPheno(options):

    print 'importing covariance matrix'
    if options.cfile is None: options.cfile=options.bfile
    XX = readCovarianceMatrixFile(options.cfile,readEig=False)['K']

    print 'simulating phenotypes'
    SP.random.seed(options.seed)
    simulator = sim.CSimulator(bfile=options.bfile,XX=XX,P=options.nTraits)
    Xr,region = simulator.getRegion(chrom_i=options.chrom,size=options.windowSize,min_nSNPs=options.nCausalR,pos_min=options.pos_min,pos_max=options.pos_max)
 
    Y,info    = genPhenoCube(simulator,Xr,vTotR=options.vTotR,nCausalR=options.nCausalR,pCommonR=options.pCommonR,vTotBg=options.vTotBg,pHidd=options.pHidden,pCommon=options.pCommon)

    print 'exporting pheno file'
    if options.pfile is not None:
        outdir = os.path.split(options.pfile)[0]
        if not os.path.exists(outdir):
            os.makedirs(outdir)
    else:
        identifier = '_seed%d_nTraits%d_wndSize%d_vTotR%.2f_nCausalR%d_pCommonR%.2f_vTotBg%.2f_pHidden%.2f_pCommon%.2f'%(options.seed,options.nTraits,options.windowSize,options.vTotR,options.nCausalR,options.pCommonR,options.vTotBg,options.pHidden,options.pCommon)
        options.pfile = os.path.split(options.bfile)[-1] + '%s'%identifier

    pfile  = options.pfile + '.phe'
    rfile  = options.pfile + '.phe.region'

    SP.savetxt(pfile,Y)
    SP.savetxt(rfile,region)



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import limix
import limix.deprecated as dlimix
from limix.test import data
import os


class CLMM_test_large(unittest.TestCase):
    """test class for CLMM"""

    def setUp(self):
        self.datasets = ['lmm_data1']
        self.dir_name = os.path.dirname(os.path.realpath(__file__))

    def test_lmm1(self):
        """basic test, comapring pv"""
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            #make vllarg X. This needs to be changed later
            NL = 1000
            self.NL = NL
            X = SP.tile(D['X'],(1,self.NL))
            lmm = dlimix.CLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(X)
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            lmm.process()
            pv = lmm.getPv().ravel()
            BetaSte = lmm.getBetaSNPste().ravel()
            Beta = lmm.getBetaSNP()
            D2pv= (SP.log10(pv)-SP.log10(SP.tile(D['pv'],self.NL))**2)
            # D2Beta= (Beta-SP.tile(D['Beta'],self.NL))**2
            # D2BetaSte = (BetaSte-SP.tile(D['BetaSte'],self.NL))**2
            RV = SP.sqrt(D2pv.mean())<1E-6
            # RV = RV & (D2Beta.mean()<1E-6)
            # RV = RV & (D2BetaSte.mean()<1E-6)
            self.assertTrue(RV)



if __name__ == '__main__':
    unittest.main()



import inspect
import numpy as np
from limix.core.type.exception import NotArrayConvertibleError


def my_name():
    return inspect.stack()[1][3]


def assert_finite_array(*arrays):
    for a in arrays:
        if not np.isfinite(a).all():
            raise ValueError("Array must not contain infs or NaNs")


def assert_make_float_array(arr, arg_name):
    try:
        arr = np.asarray(arr, dtype=float)
    except ValueError as e:
        raise NotArrayConvertibleError("%s has to be float-array "
                                       "convertible." % arg_name)
    return arr


def assert_type(arg, type_, param_name):

    err_msg = ("Parameter %s is not of type %s.%s."
               % (param_name, type_.__module__, type_.__name__))

    if type(arg) is not type_:
        raise TypeError(err_msg)


def assert_subtype(arg, type_, param_name):
    err_msg = ("Parameter %s must have %s.%s inheritance."
               % (param_name, type_.__module__, type_.__name__))

    if not issubclass(type(arg), type_):
        raise TypeError(err_msg)


def assert_type_or_list_type(arg, type_, param_name):

    err_msg = ("Parameter %s is not of type "
               "%s.%s nor a list or a tuple of the same."
               % (param_name, type_.__module__, type_.__name__))

    if type(arg) in (list, tuple):
        for a in arg:
            if type(a) is not type_:
                raise TypeError(err_msg)
    else:
        if type(arg) is not type_:
            raise TypeError(err_msg)


def assert_subtype_or_list_subtype(arg, type_, param_name):

    err_msg = ("Parameter %s is not of type "
               "%s.%s nor a list or a tuple of the same."
               % (param_name, type_.__module__, type_.__name__))

    if type(arg) in (list, tuple):
        for a in arg:
            if not issubclass(type(a),  type_):
                raise TypeError(err_msg)
    else:
        if issubclass(type(arg),  type_):
            raise TypeError(err_msg)



import sys
sys.path.append('./../../..')
import os
import subprocess
import pdb
import sys
import csv
import glob
import numpy as NP
from optparse import OptionParser
import time
import limix.stats.chi2mixture as C2M
import limix.utils.plot as plot
import scipy as SP

def plot_manhattan(pv,out_file):
    import matplotlib.pylab as PLT
    posCum = SP.arange(pv.shape[0])
    idx=~SP.isnan(pv[:,0])
    plot.plot_manhattan(posCum[idx],pv[idx][:,0],alphaNS=1.0,alphaS=1.0)
    PLT.savefig(out_file)


def postprocess(options):
    """ perform parametric fit of the test statistics and provide permutation and test pvalues """

    resdir = options.resdir
    out_file = options.outfile
    tol = options.tol

    print '.. load permutation results'
    file_name = os.path.join(resdir,'perm*','*.res')
    files = glob.glob(file_name)
    LLR0 = []
    for _file in files:
        print _file
        LLR0.append(NP.loadtxt(_file,usecols=[6]))
    LLR0 = NP.concatenate(LLR0)

    print '.. fit test statistics'
    t0 = time.time()
    c2m = C2M.Chi2mixture(tol=4e-3)
    c2m.estimate_chi2mixture(LLR0)
    pv0 = c2m.sf(LLR0)
    t1 = time.time()
    print 'finished in %s seconds'%(t1-t0)

    print '.. export permutation results'
    perm_file = out_file+'.perm'
    RV = NP.array([LLR0,pv0]).T
    NP.savetxt(perm_file,RV,delimiter='\t',fmt='%.6f %.6e')

    print '.. load test results'
    file_name = os.path.join(resdir,'test','*.res')
    files = glob.glob(file_name)
    RV_test = []
    for _file in files:
        print _file
        RV_test.append(NP.loadtxt(_file))
    RV_test = NP.concatenate(RV_test)

    print '.. calc pvalues'
    pv = c2m.sf(RV_test[:,-1])[:,NP.newaxis]

    print '.. export test results'
    perm_file = out_file+'.test'
    RV_test = NP.hstack([RV_test,pv])
    NP.savetxt(perm_file,RV_test,delimiter='\t',fmt='%d %d %d %d %d %d %.6e %.6e')

    if options.manhattan:
        manhattan_file = out_file+'.manhattan.jpg'
        plot_manhattan(pv,manhattan_file)



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import limix
import limix.deprecated as dlimix
from limix.test import data
import os


class CKroneckerLMM_test(unittest.TestCase):
    """test class for CLMM"""

    def setUp(self):
        self.datasets = ['lmm_data1']
        self.dir_name = os.path.dirname(os.path.realpath(__file__))

    def test_lmm(self):
        """basic test, comparing pv to a standard LMM equivalent"""
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            #construct Kronecker LMM model which has the special case of standard LMM
            #covar1: genotype matrix
            K1r = D['K']
            K1c = SP.eye(1)
            K2r = SP.eye(D['K'].shape[0])
            K2c = SP.eye(1)
            A   = SP.eye(1)
            Acov = SP.eye(1)
            Xcov  = D['Cov'][:,SP.newaxis]
            X      = D['X']
            Y      = D['Y'][:,SP.newaxis]

            lmm = dlimix.CKroneckerLMM()
            lmm.setK1r(K1r)
            lmm.setK1c(K1c)
            lmm.setK2r(K2r)
            lmm.setK2c(K2c)

            lmm.setSNPs(X)
            #add covariates
            lmm.addCovariates(Xcov,Acov)
            #add SNP design
            lmm.setSNPcoldesign(A)
            lmm.setPheno(Y)
            lmm.setNumIntervalsAlt(0)
            lmm.setNumIntervals0(100)

            lmm.process()
            pv = lmm.getPv().ravel()
            D2= ((SP.log10(pv)-SP.log10(D['pv']))**2)
            RV = SP.sqrt(D2.mean())
            #print "\n"
            #print pv[0:10]
            #print D['pv'][0:10]
            #print RV
            #pdb.set_trace()
            self.assertTrue(RV<1E-6)

    def test_lmm2(self):
        """another test, establishing an lmm-equivalent by a design matrix choice"""
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            #construct Kronecker LMM model which has the special case of standard LMM
            #covar1: genotype matrix
            N = D['K'].shape[0]
            P = 3
            K1r = D['K']
            #K1c = SP.zeros([2,2])
            #K1c[0,0] = 1
            K1c = SP.eye(P)
            K2r = SP.eye(N)
            K2c = SP.eye(P)

            #A   = SP.zeros([1,2])
            #A[0,0] =1
            A = SP.eye(P)
            Acov = SP.eye(P)
            Xcov = D['Cov'][:,SP.newaxis]
            X      = D['X']
            Y      = D['Y'][:,SP.newaxis]
            Y      = SP.tile(Y,(1,P))

            lmm = dlimix.CKroneckerLMM()
            lmm.setK1r(K1r)
            lmm.setK1c(K1c)
            lmm.setK2r(K2r)
            lmm.setK2c(K2c)

            lmm.setSNPs(X)
            #add covariates
            lmm.addCovariates(Xcov,Acov)
            #add SNP design
            lmm.setSNPcoldesign(A)
            lmm.setPheno(Y)
            lmm.setNumIntervalsAlt(0)
            lmm.setNumIntervals0(100)

            lmm.process()

            #get p-values with P-dof:
            pv_Pdof = lmm.getPv().ravel()
            #transform in P-values with a single DOF:
            import scipy.stats as st
            lrt = st.chi2.isf(pv_Pdof,P)/P
            pv = st.chi2.sf(lrt,1)
            #compare with single DOF P-values:
            D2= ((SP.log10(pv)-SP.log10(D['pv']))**2)
            RV = SP.sqrt(D2.mean())
            #print "\n"
            #print pv[0:10]
            #print D['pv'][0:10]
            #print RV
            #pdb.set_trace()
            self.assertTrue(RV<1E-6)



    def test_permutation(self):
        #test permutation function
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            perm = SP.random.permutation(D['X'].shape[0])
            #1. set permuattion
            lmm = dlimix.CLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            if 1:
                #pdb.set_trace()
                perm = SP.array(perm,dtype='int32')#Windows needs int32 as long -> fix interface to accept int64 types
            lmm.setPermutation(perm)
            lmm.process()
            pv_perm1 = lmm.getPv().ravel()
            #2. do by hand
            lmm = dlimix.CLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'][perm])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            lmm.process()
            pv_perm2 = lmm.getPv().ravel()
            D2 = (SP.log10(pv_perm1)-SP.log10(pv_perm2))**2
            RV = SP.sqrt(D2.mean())
            self.assertTrue(RV<1E-6)



class CInteractLMM_test:
    """Interaction test"""
    def __init__(self):
        pass

    def test_all(self):
        RV = False
        #print 'CInteractLMM IMPLEMENTED %s' % message(RV)


if __name__ == '__main__':
    unittest.main()



import sys
import numpy as np
from covar_base import Covariance
from limix.core.type.cached import cached
from limix.core.type.exception import TooExpensiveOperationError
from limix.core.utils import my_name
from util import msg_too_expensive_dim
import scipy as sp
import scipy.linalg as LA
import warnings

import pdb

_MAX_DIM = 5000

# should be a child class of combinator
class Cov2KronSum(Covariance):
    """
    Covariance class for sum of two Kronecker products
    K = Cg \kron R + Cn \kron I
    Notation:
        - dim_c: dimension of col covariances
        - dim_r: dimension of row covariances
    """

    def __init__(self, Cg = None, Cn = None, R = None, S_R = None, U_R = None):
        """
        Args:
            Cg:     Limix covariance matrix for Cg (dimension dim_c)
            Cn:     Limix covariance matrix for Cn (dimension dim_c)
            R:      [dim_r, dim_r] numpy semidemidefinite covariance matrix for R
                    In alternative to R, S_R and U_R can be specified.
            S_R:    N vector of eigenvalues of R
            U_R:    [N, N] eigenvector matrix of R
        """
        Covariance.__init__(self)
        self._Cg_act = True
        self._Cn_act = True
        self.setColCovars(Cg, Cn)
        self.setR(R=R, S_R=S_R, U_R=U_R)
        self.dim = self.dim_c * self.dim_r
        self._use_to_predict = False

    def col_covs_have_changed(self):
        self.clear_cache('col_cov')
        self._notify('col_cov')
        self.clear_all()

    def R_has_changed(self):
        self.clear_cache('row_cov')
        self._notify('row_cov')
        self.clear_all()

    #####################
    # Properties
    #####################
    @property
    def R(self):
        return self._R

    @property
    def Cg(self):
        return self._Cg

    @property
    def Cn(self):
        return self._Cn

    @property
    def dim_r(self):
        return self._dim_r

    @property
    def dim_c(self):
        return self._dim_c

    #####################
    # Setters
    #####################
    def setR(self, R=None, S_R=None, U_R=None):
        RnotNone  = R is not None
        SUnotNone = S_R is not None and U_R is not None
        assert RnotNone or SUnotNone, 'Either R or S_R and U_R must be specified.'
        if RnotNone:
            self._dim_r = R.shape[0]
            self._R = R
            self.R_has_changed()
        if SUnotNone:
            #TODO: assert dimensions
            self._dim_r = S_R.shape[0]
            self.R_has_changed()
            self.fill_cache('Lr', U_R.T)
            self.fill_cache('Sr', S_R)

    def setColCovars(self, Cg = None, Cn = None):
        assert Cg is not None, 'Cg has to be specified.'
        assert Cn is not None, 'Cn has to be specified.'
        assert Cg.dim==Cn.dim, 'Cg and Cn must have the same dimensions.'
        self._dim_c = Cg.dim
        self._Cg = Cg
        self._Cn = Cn
        self._Cg.register(self.col_covs_have_changed)
        self._Cn.register(self.col_covs_have_changed)
        self.col_covs_have_changed()

    #####################
    # Activation handling
    #####################
    @property
    def act_Cg(self):
        return self._Cg_act

    @act_Cg.setter
    def act_Cg(self, act):
        self._Cg_act = bool(act)
        self._notify()

    @property
    def act_Cn(self):
        return self._Cn_act

    @act_Cn.setter
    def act_Cn(self, act):
        self._Cn_act = bool(act)
        self._notify()

    #####################
    # Params handling
    #####################
    def setParams(self, params):
        nCg = self.Cg.getNumberParams()
        nCn = self.Cn.getNumberParams()
        nact = nCg * int(self._Cg_act) + nCn * int(self._Cn_act)

        if len(params) != nact:
            raise ValueError("The number of parameters passed to setParams "
                             "differs from the number of active parameters.")

        self.Cg.setParams(params[:nCg])
        self.Cn.setParams(params[nCg:])

    def getParams(self):
        params = []
        if self._Cg_act:
            params.append(self.Cg.getParams())
        if self._Cn_act:
            params.append(self.Cn.getParams())
        if len(params) == 0:
            return np.array([])
        return sp.concatenate(params)

    def getNumberParams(self):
        return (int(self._Cg_act) * self.Cg.getNumberParams() +
                int(self._Cn_act) * self.Cn.getNumberParams())

    def _actindex2index(self, i):
        nCg = self.Cg.getNumberParams()
        i += nCg * int(not self._Cg_act)
        return i

    def _index2actindex(self, i):
        nCg = self.Cg.getNumberParams()
        i -= nCg * int(not self._Cg_act)
        return i

    #####################
    # Cached
    #####################
    @cached('col_cov')
    def Cstar(self):
        return sp.dot(self.Cn.USi2().T,sp.dot(self.Cg.K(),self.Cn.USi2()))

    @cached('col_cov')
    def S_Cstar(self):
        RV,U = LA.eigh(self.Cstar())
        self.fill_cache('U_Cstar',U)
        return RV

    @cached('col_cov')
    def U_Cstar(self):
        S,RV = LA.eigh(self.Cstar())
        self.fill_cache('S_Cstar',S)
        return RV

    @cached(['col_cov', 'row_cov'])
    def SpI(self):
        return sp.kron(self.S_Cstar(),self.Sr())+1

    @cached(['col_cov', 'row_cov'])
    def d(self):
        return 1./self.SpI()

    @cached(['col_cov', 'row_cov'])
    def D(self):
        return self.d().reshape((self._dim_r, self._dim_c), order = 'F')

    @cached('col_cov')
    def Lc(self):
        return sp.dot(self.U_Cstar().T,self.Cn.USi2().T)

    @cached('row_cov')
    def Sr(self):
        RV, U = LA.eigh(self.R, check_finite=False)
        self.fill_cache('Lr',U.T)
        return RV

    @cached('row_cov')
    def Lr(self):
        S,U = LA.eigh(self.R)
        self.fill_cache('Sr',S)
        return U.T

    @cached('col_cov')
    def LcGradCgLc(self, i):
        return sp.dot(self.Lc(), sp.dot(self.Cg.K_grad_i(i), self.Lc().T))

    @cached('col_cov')
    def LcGradCnLc(self, i):
        return sp.dot(self.Lc(), sp.dot(self.Cn.K_grad_i(i), self.Lc().T))

    @cached('col_cov')
    def Ctilde(self, i):
        if i < self.Cg.getNumberParams():
            r = self.LcGradCgLc(i)
        else:
            _i = i - self.Cg.getNumberParams()
            r = self.LcGradCnLc(_i)
        return r

    def Sr_X_Ctilde(self, X, i):
        if i < self.Cg.getNumberParams():
            SrX = self.Sr()[:, sp.newaxis] * X
            r = sp.dot(SrX, self.LcGradCgLc(i).T)
        else:
            _i = i - self.Cg.getNumberParams()
            r = sp.dot(X, self.LcGradCnLc(_i).T)
        return r

    def diag_Ctilde_o_Sr(self, i):
        if i < self.Cg.getNumberParams():
            r = sp.kron(sp.diag(self.LcGradCgLc(i)), self.Sr())
        else:
            _i = i - self.Cg.getNumberParams()
            r = sp.kron(sp.diag(self.LcGradCnLc(_i)), sp.ones(self.dim_r))
        return r

    @cached(['row_cov', 'col_cov'])
    def L(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        return sp.kron(self.Lc(), self.Lr())

    #####################
    # Overwritten covar_base methods
    #####################
    @cached(['row_cov', 'col_cov', 'covar_base'])
    def K(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        rv = sp.kron(self.Cg.K(), self.R) + sp.kron(self.Cn.K(), sp.eye(self.dim_r))
        return rv

    @cached(['row_cov', 'col_cov', 'covar_base'])
    def K_grad_i(self,i):
        nCg = self.Cg.getNumberParams()

        if i >= self.getNumberParams():
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")

        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        i = self._actindex2index(i)

        if i < nCg:
            rv= sp.kron(self.Cg.K_grad_i(i), self.R)
        else:
            _i = i - nCg
            rv = sp.kron(self.Cn.K_grad_i(_i), sp.eye(self.dim_r))
        return rv

    @cached(['row_cov', 'col_cov', 'covar_base'])
    def logdet(self):
        return sp.sum(sp.log(self.Cn.S())) * self.dim_r + sp.log(self.SpI()).sum()

    @cached(['row_cov', 'col_cov', 'covar_base'])
    def logdet_grad_i(self,i):
        if i >= self.getNumberParams():
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")
        i = self._actindex2index(i)
        return (self.d() * self.diag_Ctilde_o_Sr(i)).sum()

""" Gradients DEPRECATED """

if 0:

    def CstarGrad_g(self,i):
        return sp.dot(self.Cn.USi2().T,sp.dot(self.Cg.Kgrad_param(i),self.Cn.USi2()))

    def CstarGrad_n(self,i):
        RV = sp.dot(self.Cn.USi2grad(i).T,sp.dot(self.Cg.K(),self.Cn.USi2()))
        RV+= sp.dot(self.Cn.USi2().T,sp.dot(self.Cg.K(),self.Cn.USi2grad(i)))
        #RV+= RV.T
        return RV

    def S_CstarGrad_g(self,i):
        return dS_dti(self.CstarGrad_g(i),U=self.U_Cstar())

    def U_CstarGrad_g(self,i):
        return dU_dti(self.CstarGrad_g(i),U=self.U_Cstar(),S=self.S_Cstar())

    def S_CstarGrad_n(self,i):
        return dS_dti(self.CstarGrad_n(i),U=self.U_Cstar())

    def U_CstarGrad_n(self,i):
        return dU_dti(self.CstarGrad_n(i),U=self.U_Cstar(),S=self.S_Cstar())

    def LcGrad_g(self,i):
        return sp.dot(self.U_CstarGrad_g(i).T,self.Cn.USi2().T)

    def LcGrad_n(self,i):
        RV  = sp.dot(self.U_CstarGrad_n(i).T,self.Cn.USi2().T)
        RV += sp.dot(self.U_Cstar().T,self.Cn.USi2grad(i).T)
        return RV

    def Sgrad_g(self,i):
        return sp.kron(self.S_CstarGrad_g(i),self.Sr())

    def Sgrad_n(self,i):
        return sp.kron(self.S_CstarGrad_n(i),self.Sr())


if __name__ == '__main__':
    from limix.core.covar import FreeFormCov
    from limix.utils.preprocess import covar_rescale

    # define row caoriance
    dim_r = 10
    X = sp.rand(dim_r, dim_r)
    R = covar_rescale(sp.dot(X,X.T))

    # define col covariances
    dim_c = 3
    Cg = FreeFormCov(dim_c)
    Cn = FreeFormCov(dim_c)

    cov = Cov2KronSum(Cg = Cg, Cn = Cn, R = R)
    cov.setRandomParams()

    print cov.K()
    print cov.K_grad_i(0)



import unittest

from limix.core.mean import MeanKronSum
from limix.core.covar import FreeFormCov
from limix.core.covar import FixedCov
from limix.core.covar import KronCov 
from limix.core.covar.combinators import SumCov
from limix.core.gp.gp_base import GP
from limix.core.gp import GP2KronSum
from limix.utils.check_grad import mcheck_grad
from limix.utils.preprocess import covar_rescale

import copy
import numpy as np
import scipy as sp


class TestGPBase(unittest.TestCase):
    def setUp(self):
        np.random.seed(1)

        # define phenotype
        N = 10 
        P = 3
        Y = sp.randn(N,P)

        # pheno with missing data
        Ym = Y.copy()
        Im = sp.rand(N, P)<0.2
        Ym[Im] = sp.nan

        # define fixed effects
        F = []; A = []
        F.append(1.*(sp.rand(N,2)<0.5))
        A.append(sp.eye(P))
        mean = MeanKronSum(Y, F=F, A=A)
        mean_m = MeanKronSum(Ym, F=F, A=A)

        # define row caoriance
        f = 10
        X = 1.*(sp.rand(N, f)<0.2)
        R = covar_rescale(sp.dot(X,X.T))
        R+= 1e-4 * sp.eye(N)

        # define col covariances
        Cg = FreeFormCov(P)
        Cn = FreeFormCov(P)
        Cg.setRandomParams()
        Cn.setRandomParams()

        # define covariance matrices
        covar1 = KronCov(Cg, R)
        covar2 = KronCov(Cn, sp.eye(N))
        covar  = SumCov(covar1,covar2)

        # define covariance matrice with missing data
        Iok = (~Im).reshape(N * P, order='F')
        covar1_m = KronCov(copy.copy(Cg), R, Iok=Iok)
        covar2_m = KronCov(copy.copy(Cn), sp.eye(N), Iok=Iok)
        covar_m  = SumCov(covar1_m,covar2_m)

        # define gp
        self._gp = GP(covar=covar, mean=mean)
        self._gpm = GP(covar=covar_m, mean=mean_m)
        self._gp2ks = GP2KronSum(Y=Y, F=F, A=A, Cg=Cg, Cn=Cn, R=R) 

    def test_gpbase_gp2kronSum(self): 
        d1 = self._gp2ks.LML()-self._gp.LML()
        d2 = self._gp2ks.LML_grad()['covar']
        d2-= self._gp.LML_grad()['covar']
        d  = sp.concatenate([sp.array([d1]), d2])
        np.testing.assert_almost_equal(d, 0., decimal=8)
        

    def test_grad_gpbase_missdata(self):

        gp = self._gpm

        def func(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            return gp.LML()

        def grad(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            grad = gp.LML_grad()
            return grad['covar'][i]

        x0 = gp.getParams()['covar']
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=5)

if __name__ == "__main__":
    unittest.main()



import sys
from limix.core.mean import MeanKronSum
from limix.core.covar import Cov2KronSumLR
from limix.core.covar import Covariance
from limix.core.type.cached import *

import pdb
import numpy as NP
import scipy as sp
import scipy.linalg as LA
import time as TIME
from limix.core.gp import GP
from limix.core.covar.cov_reml import cov_reml
from limix.utils.util_functions import vec
from limix.core.utils import assert_type_or_list_type
from limix.core.utils import assert_type
from limix.core.utils import assert_subtype


class GP2KronSumLR(GP):
    """
    Gaussian Process with a 2kronSumLR Covariance and a mean that is a sum of Kronecker products:
        vec(Y) ~ N( vec( \sum_i F_i B_i A_i), Cr \kron GG.T + Cn \kron I )
    The current implementation supports only 1 fixed effect term!
    Notation:
        N = number of samples
        P = number of traits
        Y = [N, P] phenotype matrix
        F_i = sample fixed effect design for term i
        A_i = trait fixed effect design for term i
        B_i = effect sizes of fixed effect term i
        Cr = column covariance matrix for low-rank term
        Cn = column covariance matrix for noise term
        rank_c = rank of low-rank col covariance
        rank_r = rank of low-rank row covariance
    """

    def __init__(self, Y, Cn, G, F=None, A=None, rank=1):
        """
        Args:
            Y:      [N, P] phenotype matrix
            F:      Sample fixed effect design (first dimension must be N)
            A:      Trait fixed effect design (second dimension must be P)
            Cn:     Limix covariance matrix for Cn (dimension P)
            G:      [N, rank_r] numpy covariance matrix for G
            rank:   rank of column low-rank covariance (default = 1)
        """
        assert_type(Y, NP.ndarray, 'Y')
        assert_subtype(Cn, Covariance, 'Cn')
        assert_type(G, NP.ndarray, 'G')

        covar = Cov2KronSumLR(Cn=Cn, G=G, rank=rank)
        covar.setRandomParams()
        mean = MeanKronSum(Y=Y, F=F, A=A)
        assert mean.n_terms <= 1, ('GP2KronSum supports MeanKronSum'
                                   ' means with maximum 1 term!')
        GP.__init__(self, covar=covar, mean=mean)

    def _observe(self):
        self.covar.register(self.row_cov_has_changed, 'row_cov')
        self.covar.register(self.col_cov_has_changed, 'col_cov')
        self.mean.register(self.pheno_has_changed, 'pheno')
        self.mean.register(self.designs_have_changed, 'designs')

    def col_cov_has_changed(self):
        self.clear_cache('col_cov')
        self.clear_all()

    def row_cov_has_changed(self):
        self.clear_cache('row_cov')
        self.clear_all()

    def pheno_has_changed(self):
        self.clear_cache('pheno')
        self.clear_all()

    def designs_have_changed(self):
        self.clear_cache('designs')
        self.clear_all()

    ######################
    # Transformed phenotype
    ######################
    @cached(['pheno'])
    def YY(self):
        return sp.dot(self.mean.Y.T, self.mean.Y)

    @cached(['designs', 'pheno'])
    def FY(self):
        return sp.dot(self.mean.F[0].T, self.mean.Y)

    @cached(['row_cov', 'pheno'])
    def WrY(self):
        return sp.dot(self.covar.Wr(), self.mean.Y)

    @cached(['row_cov', 'pheno'])
    def GGY(self):
        return sp.dot(self.covar.G, sp.dot(self.covar.G.T, self.mean.Y))

    @cached(['row_cov', 'pheno'])
    def YGGY(self):
        return sp.dot(self.mean.Y.T, self.GGY()) 

    @cached(['row_cov', 'pheno'])
    def WrGGY(self):
        return sp.dot(self.covar.Wr(), self.GGY())

    @cached(['col_cov', 'pheno'])
    def YLc(self):
        return sp.dot(self.mean.Y, self.covar.Lc().T)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def WrYLc(self):
        return sp.dot(self.WrY(), self.covar.Lc().T)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def WrYLcWc(self):
        return sp.dot(self.WrYLc(), self.covar.Wc().T)

    @cached(['designs', 'col_cov', 'pheno'])
    def FYLc(self):
        return sp.dot(self.FY(), self.covar.Lc().T)

    @cached(['designs', 'col_cov', 'pheno'])
    def FYLcALc(self):
        return sp.dot(self.FYLc(), self.ALc().T)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def GGYLc(self):
        return sp.dot(self.GGY(), self.covar.Lc().T)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def WrGGYLc(self):
        return sp.dot(self.WrGGY(), self.covar.Lc().T)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def DWrYLcWc(self):
        return self.covar.D() * self.WrYLcWc()

    #@cached(['row_cov', 'col_cov', 'pheno'])
    #def RYLcCtilde(self, i):
    #    if i < self.covar.Cr.getNumberParams():
    #        RYLc = self.GGYLc()
    #    else:
    #        RYLc = self.YLc()
    #    return sp.dot(RYLc, self.covar.Ctilde(i))

    @cached(['row_cov', 'col_cov', 'pheno'])
    def SrDWrYLcWcCbar(self, i):
        if i < self.covar.Cr.getNumberParams():
            SgDWrYLcWc = self.covar.Sg()[:, sp.newaxis] *  self.DWrYLcWc()
        else:
            SgDWrYLcWc = self.DWrYLcWc()
        return sp.dot(SgDWrYLcWc, self.covar.Cbar(i))

    def YRY(self, i):
        if i < self.covar.Cr.getNumberParams():
            return self.YGGY()
        else:
            return self.YY()

    @cached(['row_cov', 'col_cov', 'pheno'])
    def WrRYLcCtildeWc(self, i):
        if i < self.covar.Cr.getNumberParams():
            WrRYLc = self.WrGGYLc()
        else:
            WrRYLc = self.WrYLc()
        return sp.dot(WrRYLc, sp.dot(self.covar.Ctilde(i), self.covar.Wc().T))

    ###############################################
    # Fixed effects
    # uses that MeanKronSum has only one term
    ###############################################
    @cached(['designs', 'row_cov'])
    def WrF(self):
        return sp.dot(self.covar.Wr(), self.mean.F[0])

    @cached(['designs', 'row_cov'])
    def GGF(self):
        return sp.dot(self.covar.G, sp.dot(self.covar.G.T, self.mean.F[0]))

    @cached(['designs', 'row_cov'])
    def YGGF(self):
        return sp.dot(self.mean.Y.T, self.GGF()) 

    @cached(['designs', 'row_cov'])
    def FGGF(self):
        return sp.dot(self.mean.F[0].T, self.GGF())

    @cached('designs')
    def FF(self):
        return sp.dot(self.mean.F[0].T, self.mean.F[0])

    @cached(['designs', 'row_cov'])
    def WrGGF(self):
        return sp.dot(self.covar.Wr(), self.GGF())

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def FB(self):
        return sp.dot(self.mean.F[0], self.mean.B[0])

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def YGGFBA(self):
        return sp.dot(sp.dot(self.YGGF(), self.mean.B[0]), self.mean.A[0]) 

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def YFBA(self):
        return sp.dot(sp.dot(self.FY().T, self.mean.B[0]), self.mean.A[0]) 

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrFB(self):
        return sp.dot(self.WrF(), self.mean.B[0])

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def GGFB(self):
        return sp.dot(self.GGF(), self.mean.B[0])

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrGGFB(self):
        return sp.dot(self.WrGGF(), self.mean.B[0])

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def FBALc(self):
        return sp.dot(self.FB(), self.ALc())

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrFBALc(self):
        return sp.dot(self.WrFB(), self.ALc())

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def GGFBALc(self):
        return sp.dot(self.GGFB(), self.ALc())

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrGGFBALc(self):
        return sp.dot(self.WrGGFB(), self.ALc())

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrGGFBALcWc(self):
        return sp.dot(self.WrGGFBALc(), self.covar.Wc().T)

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrFBALcWc(self):
        return sp.dot(self.WrFBALc(), self.covar.Wc().T)

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def DWrGGFBALcWc(self):
        return self.covar.D() * self.WrGGFBALcWc()

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def DWrFBALcWc(self):
        return self.covar.D() * self.WrFBALcWc()

    @cached(['designs', 'col_cov'])
    def ALc(self):
        return sp.dot(self.mean.A[0], self.covar.Lc().T)

    @cached(['designs', 'col_cov'])
    def ALcLcA(self):
        return sp.dot(self.ALc(), self.ALc().T)

    @cached(['designs', 'col_cov'])
    def ALcWc(self):
        return sp.dot(self.ALc(), self.covar.Wc().T)

    @cached(['designs', 'row_cov', 'col_cov'])
    def WLW(self):
        # confusing notation the first W refers to kroneckered matrix
        # that brings to the low-dimensional space while the second refers to the fixed effect design
        return sp.kron(self.ALcWc().T, self.WrF())

    @cached(['designs', 'row_cov', 'col_cov'])
    def dWLW(self):
        return self.covar.d()[:,sp.newaxis] * self.WLW()

    #@cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    #def RFBALcCtilde(self, i):
    #    if i < self.covar.Cr.getNumberParams():
    #        RFBALc = self.GGFBALc()
    #    else:
    #        RFBALc = self.FBALc()
    #    return sp.dot(RFBALc, self.covar.Ctilde(i))

    def YRFBA(self, i):
        if i < self.covar.Cr.getNumberParams():
            return self.YGGFBA()
        else:
            return self.YFBA()

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def SrDWrFBALcWcCbar(self, i):
        if i < self.covar.Cr.getNumberParams():
            SgDWrFBALcWc = self.covar.Sg()[:, sp.newaxis] *  self.DWrFBALcWc()
        else:
            SgDWrFBALcWc = self.DWrFBALcWc()
        return sp.dot(SgDWrFBALcWc, self.covar.Cbar(i))

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WrRFBALcCtildeWc(self, i):
        if i < self.covar.Cr.getNumberParams():
            WrRFBALc = self.WrGGFBALc()
        else:
            WrRFBALc = self.WrFBALc()
        return sp.dot(WrRFBALc, sp.dot(self.covar.Ctilde(i), self.covar.Wc().T))

    # fixed effect methods for Areml_grad

    @cached('col_cov')
    def CtildeLcA(self, i):
        return sp.dot(self.covar.Ctilde(i), self.ALc().T)

    @cached(['designs', 'row_cov', 'col_cov'])
    def ALcCtildeLcA_o_FRF(self, i):
        if i < self.covar.Cr.getNumberParams():
            FRF = self.FGGF()
        else:
            FRF = self.FF()
        ALcCtildeALc = sp.dot(self.ALc(), self.CtildeLcA(i))
        return sp.kron(ALcCtildeALc, FRF)

    @cached(['designs', 'row_cov', 'col_cov'])
    def Cbar_o_Sr_dWLW(self, i):
        if i < self.covar.Cr.getNumberParams():
            Cbar_o_Sr = sp.kron(self.covar.Cbar(i), sp.diag(self.covar.Sg()))
        else:
            Cbar_o_Sr = sp.kron(self.covar.Cbar(i), sp.eye(self.covar.rank_r))
        return sp.dot(Cbar_o_Sr, self.dWLW())

    @cached(['designs', 'row_cov', 'col_cov'])
    def WcCtildeLcA_o_WrRF(self, i):
        if i < self.covar.Cr.getNumberParams():
            self.WrRF = self.WrGGF()
        else:
            self.WrRF = self.WrF()
        WcCtildeLcA = sp.dot(self.covar.Wc(), self.CtildeLcA(i))
        return sp.kron(WcCtildeLcA, self.WrRF)


    ######################
    # Areml
    ######################
    def Areml_K(self):
        R  = sp.kron(self.ALcLcA(), self.FF())
        R -= sp.dot(self.WLW().T, self.dWLW())
        return R

    def Areml_K_grad_i(self,i):
        i = self.covar._actindex2index(i)
        R = sp.dot(self.WcCtildeLcA_o_WrRF(i).T, self.dWLW())
        R+= R.T
        R+= -self.ALcCtildeLcA_o_FRF(i)
        R+= -sp.dot(self.dWLW().T, self.Cbar_o_Sr_dWLW(i))
        return R

    ########################
    # LML terms
    ########################
    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def WKiy(self):
        R = vec(self.FYLcALc())
        # the following could be further optimized but do not care for now
        R -= sp.dot(self.WLW().T, vec(self.DWrYLcWc()))
        return R

    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def b(self):
        if self.mean.n_covs > 0:
            R = self.Areml.solve(self.WKiy())
        else:
            R = None
        return R

    @cached(['row_cov', 'col_cov', 'pheno'])
    def yKiy(self):
        return (self.YLc() * self.YLc()).sum() - (self.WrYLcWc() * self.DWrYLcWc()).sum()

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def yKiWb(self):
        return (self.WKiy() * self.mean.b).sum()

    #########################
    # Gradients
    #########################
    @cached(['row_cov', 'col_cov', 'pheno'])
    def yKiy_grad_i(self,i):
        i = self.covar._actindex2index(i)
        r = - (self.YRY(i).T * self.covar.LcCtildeLc(i)).sum()
        r+= - (self.DWrYLcWc() * self.SrDWrYLcWcCbar(i)).sum()
        r+= 2 * (self.WrRYLcCtildeWc(i) * self.DWrYLcWc()).sum()
        return r

    @cached(['designs', 'row_cov', 'col_cov', 'pheno'])
    def yKiWb_grad_i(self,i):
        Areml_grad_i_b = sp.dot(self.Areml.K_grad_i(i), self.mean.b) 
        i = self.covar._actindex2index(i)
        #r = -2 * (self.YLc() * self.RFBALcCtilde(i)).sum()
        r = - 2 * (self.YRFBA(i).T * self.covar.LcCtildeLc(i)).sum()
        r+= -2 * (self.DWrYLcWc() * self.SrDWrFBALcWcCbar(i)).sum()
        r+= 2 * (self.WrRYLcCtildeWc(i) * self.DWrFBALcWc()).sum()
        r+= 2 * (self.WrRFBALcCtildeWc(i) * self.DWrYLcWc()).sum()
        r-= (self.mean.b * Areml_grad_i_b).sum()
        #r+= (self.FBALc() * self.RFBALcCtilde(i)).sum()
        #r+= (self.DWrFBALcWc() * self.SrDWrFBALcWcCbar(i)).sum()
        #r+= - 2 * (self.WrRFBALcCtildeWc(i) * self.DWrFBALcWc()).sum()
        return r



import sys
sys.path.insert(0,'./../../..')
from limix.core.cobj import *
from limix.utils.preprocess import regressOut
import scipy as sp
import numpy as np

import scipy.linalg as LA
import copy
import pdb

class Base(cObject):

    def __init__(self,Y,F):
        """
        y:        phenotype vector
        F:        fixed effect design
        """
        self.Y = Y
        self.F = F
        self.B = sp.zeros((self._K,1))

    #########################################
    # Properties
    #########################################
    @property
    def Y(self):
        return self._Y

    @property
    def F(self):
        return self._F

    @property
    def B(self):
        return self._B

    #########################################
    # Setters
    #########################################
    @Y.setter
    def Y(self,value):
        """ set phenotype """
        assert value.shape[1]==1, 'Dimension mismatch'
        self._N = value.shape[0]
        self._Y = value
        #self.clear_cache()

    @F.setter
    def F(self,value):
        """ set phenotype """
        assert value.shape[0]==self._N, 'Dimension mismatch'
        self._K = value.shape[1]
        self._F = value
        self.clear_cache('predict','Yres')

    @B.setter
    def B(self,value):
        """ set phenotype """
        assert value.shape[0]==self._K, 'Dimension mismatch'
        assert value.shape[1]==1, 'Dimension mismatch'
        self._B = value
        self.clear_cache('predict','Yres')

    #########################################
    # Getters (caching)
    #########################################
    @cached
    def predict(self):
        """ predict the value of the fixed effect (F*B) """
        return sp.dot(self.F,self.B)

    @cached
    def Yres(self):
        """ residual """
        RV  = self.Y-self.predict()
        return RV

    ###########################################
    # Gradient TODO
    ###########################################
    #def getGradient(self,j):
    #    """ get gradient for fixed effect i """
    #    return rv

    #########################################
    # Params manipulation TODO
    #########################################
    #def getParams(self):
    #    """ get params """
    #    return rv

    #def setParams(self,params):
    #    """ set params """
    #    start = 0
    #    for i in range(self.n_terms):
    #        n_effects = self.B[i].size
    #        self.B[i] = np.reshape(params[start:start+n_effects],self.B[i].shape, order='F')
    #        start += n_effects



from fixed import FixedCov
from sqexp import SQExpCov
from freeform import FreeFormCov
from lowrank import LowRankCov
from combinators import SumCov
from kronecker import KronCov
from cov2kronSum import Cov2KronSum
from cov2kronSumLR import Cov2KronSumLR
from cov3kronSumLR import Cov3KronSumLR
from covar_base import Covariance



import sys
from limix.core.mean import MeanKronSum
from limix.core.covar import Cov2KronSum
from limix.core.covar import Covariance
from limix.core.type.cached import *

import pdb
import numpy as NP
import scipy as sp
import scipy.linalg as LA
import time as TIME
from gp_base import GP
from limix.core.covar.cov_reml import cov_reml
from limix.core.utils import assert_type_or_list_type
from limix.core.utils import assert_type
from limix.core.utils import assert_subtype


class GP2KronSum(GP):
    """
    Gaussian Process with a 2kronSum Covariance and a mean that is a sum of Kronecker products:
        vec(Y) ~ N( vec( \sum_i F_i B_i A_i), Cg \kron R + Cn \kron I )
    Notation:
        N = number of samples
        P = number of traits
        Y = [N, P] phenotype matrix
        F_i = sample fixed effect design for term i
        A_i = trait fixed effect design for term i
        B_i = effect sizes of fixed effect term i
        Cg = column covariance matrix for signal term respectively
        Cn = column covariance matrix for noise term respectively
        R = row covariance matrix for signal term respectively
    """

    def __init__(self, Y, Cg, Cn, F=None, A=None, R=None, S_R=None, U_R=None):
        """
        Args:
            Y:      [N, P] phenotype matrix
            F:      list of sample fixed effect designs.
                    Each term must have first dimension N
            A:      list of trait fixed effect design.
                    Each term must have second dimension P
            Cg:     Limix covariance matrix for Cg (dimension P)
            Cn:     Limix covariance matrix for Cn (dimension P)
            R:      [N, N] numpy semidemidefinite covariance matrix for R.
                    In alternative to R, S_R and U_R can be specified.
            S_R:    N vector of eigenvalues of R
            U_R:    [N, N] eigenvector matrix of R
        """
        assert_type(Y, NP.ndarray, 'Y')
        assert_subtype(Cg, Covariance, 'Cg')
        assert_subtype(Cn, Covariance, 'Cn')

        covar = Cov2KronSum(Cg=Cg, Cn=Cn, R=R, S_R=S_R, U_R=U_R)
        mean = MeanKronSum(Y=Y, F=F, A=A)

        GP.__init__(self, covar=covar, mean=mean)

    def _observe(self):
        self.covar.register(self.row_cov_has_changed, 'row_cov')
        self.covar.register(self.col_cov_has_changed, 'col_cov')
        self.mean.register(self.pheno_has_changed, 'pheno')
        self.mean.register(self.designs_have_changed, 'designs')

    def col_cov_has_changed(self):
        self.clear_cache('col_cov')
        self.clear_all()

    def row_cov_has_changed(self):
        self.clear_cache('row_cov')
        self.clear_all()

    def pheno_has_changed(self):
        self.clear_cache('pheno')
        self.clear_all()

    def designs_have_changed(self):
        self.clear_cache('designs')
        self.clear_all()


    ######################
    # Transformed phenotype
    ######################
    @cached('row_cov', 'pheno')
    def LrY(self):
        return sp.dot(self.covar.Lr(), self.mean.Y)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def LrYLc(self):
        return sp.dot(self.LrY(), self.covar.Lc().T)

    @cached(['row_cov', 'col_cov', 'pheno'])
    def DLrYLc(self):
        return self.covar.D() * self.LrYLc()

    ######################
    # Transformed fixed effects
    ######################
    @cached(['row_cov', 'designs'])
    def LrF(self):
        R = []
        for ti in range(self.mean.n_terms):
            R.append(sp.dot(self.covar.Lr(), self.mean.F[ti]))
        return R

    @cached(['col_cov', 'designs'])
    def ALc(self):
        R = []
        for ti in range(self.mean.n_terms):
            R.append(sp.dot(self.mean.A[ti], self.covar.Lc().T))
        return R

    @cached(['row_cov', 'col_cov', 'designs'])
    def LW(self):
        R = sp.zeros((self.mean.Y.size, self.mean.n_covs))
        istart = 0
        for ti in range(self.mean.n_terms):
            iend = istart + self.mean.F[ti].shape[1] * self.mean.A[ti].shape[0]
            R[:, istart:iend] = sp.kron(self.ALc()[ti].T, self.LrF()[ti])
        return R

    @cached(['row_cov', 'col_cov', 'designs'])
    def dLW(self):
        return self.covar.d()[:,sp.newaxis] * self.LW()

    @cached(['row_cov', 'col_cov', 'pheno'])
    def Sr_DLrYLc_Ctilde(self, i):
        return self.covar.Sr_X_Ctilde(self.DLrYLc(), i)

    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def Sr_vei_dLWb_Ctilde(self, i):
        return self.covar.Sr_X_Ctilde(self.vei_dLWb(), i)

    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def vei_dLWb(self):
        # could be optimized but probably not worth it
        # as it requires a for loop
        r = sp.dot(self.dLW(), self.mean.b)
        return r.reshape(self.mean.Y.shape, order = 'F')

    ######################
    # Areml
    ######################
    def Areml_K(self):
        return sp.dot(self.LW().T, self.dLW())

    def Areml_K_grad_i(self,i):
        dLWt = self.dLW().reshape((self.mean._N, self.mean._P, self.mean.n_covs), order = 'F')
        if i < self.covar.Cg.getNumberParams():
            SrdLWt = self.covar.Sr()[:, sp.newaxis, sp.newaxis] * dLWt
        else:
            SrdLWt = dLWt
        SrdLWtC = sp.tensordot(SrdLWt, self.covar.Ctilde(i), axes=(1, 1))
        SroCdLW = SrdLWtC.swapaxes(1,2).reshape((self.mean._N * self.mean._P, self.mean.n_covs), order = 'F')
        return -sp.dot(self.dLW().T, SroCdLW)

    ########################
    # LML terms
    ########################
    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def WKiy(self):
        R = sp.zeros((self.mean.n_covs, 1))
        istart = 0
        for ti in range(self.mean.n_terms):
            _dim = self.mean.F[ti].shape[1] * self.mean.A[ti].shape[0]
            iend = istart + _dim
            FLrDLrYLc = sp.dot(self.LrF()[ti].T, self.DLrYLc())
            R[istart:iend, 0] = sp.dot(FLrDLrYLc, self.ALc()[ti].T).reshape(_dim, order = 'F')
        return R

    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def b(self):
        if self.mean.n_covs > 0:
            R = self.Areml.solve(self.WKiy())
        else:
            R = None
        return R

    @cached(['row_cov', 'col_cov', 'pheno'])
    def yKiy(self):
        return (self.LrYLc()*self.DLrYLc()).sum()

    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def yKiWb(self):
        return (self.LrYLc() * self.vei_dLWb()).sum()

    #########################
    # Gradients
    #########################
    @cached(['row_cov', 'col_cov', 'pheno'])
    def yKiy_grad_i(self,i):
        return -(self.DLrYLc() * self.Sr_DLrYLc_Ctilde(i)).sum()

    @cached(['row_cov', 'col_cov', 'designs', 'pheno'])
    def yKiWb_grad_i(self,i):
        rv = -2*(self.DLrYLc()*self.Sr_vei_dLWb_Ctilde(i)).sum()
        rv+= (self.vei_dLWb()*self.Sr_vei_dLWb_Ctilde(i)).sum()
        return rv



import unittest
import numpy as np
from limix.core.covar import SQExpCov
from limix.utils.check_grad import mcheck_grad
from limix.core.type.exception import NotArrayConvertibleError
import scipy as sp

class TestSQExp(unittest.TestCase):
    def setUp(self):
        np.random.seed(1)
        self._X = np.random.randn(10, 5)
        self._cov = SQExpCov(self._X)

    def test_setX_retE(self):
        X1 = self._X
        np.random.seed(2)
        X2 = np.random.randn(10, 5)

        E1 = sp.spatial.distance.pdist(X1,'euclidean')**2
        E1 = sp.spatial.distance.squareform(E1)

        E2 = sp.spatial.distance.pdist(X2,'euclidean')**2
        E2 = sp.spatial.distance.squareform(E2)

        np.testing.assert_almost_equal(E1, self._cov.E())

        self._cov.X = X2
        np.testing.assert_almost_equal(E2, self._cov.E())

    def test_param_activation(self):
        self._cov.act_scale = False
        self._cov.act_length = False
        self.assertEqual(len(self._cov.getParams()), 0)

        self._cov.act_scale = False
        self._cov.act_length = True
        self.assertEqual(len(self._cov.getParams()), 1)

        self._cov.act_scale = True
        self._cov.act_length = False
        self.assertEqual(len(self._cov.getParams()), 1)

        self._cov.act_scale = True
        self._cov.act_length = True
        self.assertEqual(len(self._cov.getParams()), 2)

        self._cov.act_scale = False
        self._cov.act_length = False
        self._cov.setParams(np.array([]))
        with self.assertRaises(ValueError):
            self._cov.setParams(np.array([0]))

        with self.assertRaises(ValueError):
            self._cov.K_grad_i(0)

        with self.assertRaises(ValueError):
            self._cov.K_grad_i(1)

    def test_Kgrad(self):

        def func(x, i):
            self._cov.scale = x[i]
            return self._cov.K()

        def grad(x, i):
            self._cov.scale = x[i]
            return self._cov.K_grad_i(0)

        x0 = np.array([self._cov.scale])
        err = mcheck_grad(func, grad, x0)

        np.testing.assert_almost_equal(err, 0.)

        def func(x, i):
            self._cov.length = x[i]
            return self._cov.K()

        def grad(x, i):
            self._cov.scale = x[i]
            return self._cov.K_grad_i(1)

        x0 = np.array([self._cov.scale])
        err = mcheck_grad(func, grad, x0)

    def test_Kgrad_activation(self):
        self._cov.act_length = False
        def func(x, i):
            self._cov.scale = x[i]
            return self._cov.K()

        def grad(x, i):
            self._cov.scale = x[i]
            return self._cov.K_grad_i(0)

        x0 = np.array([self._cov.scale])
        err = mcheck_grad(func, grad, x0)

        np.testing.assert_almost_equal(err, 0.)

        self._cov.act_scale = False
        self._cov.act_length = True
        def func(x, i):
            self._cov.length = x[i]
            return self._cov.K()

        def grad(x, i):
            self._cov.length = x[i]
            return self._cov.K_grad_i(0)

        x0 = np.array([self._cov.length])
        err = mcheck_grad(func, grad, x0)

        np.testing.assert_almost_equal(err, 0.)

    def test_Khess(self):

        def func(x, i):
            self._cov.scale = x[i]
            return self._cov.K_grad_i(0)

        def grad(x, i):
            self._cov.scale = x[i]
            return self._cov.K_hess_i_j(0, 0)

        x0 = np.array([self._cov.scale])
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=5)

        def func(x, i):
            self._cov.length = x[i]
            return self._cov.K_grad_i(0)

        def grad(x, i):
            self._cov.length = x[i]
            return self._cov.K_hess_i_j(0, 1)

        x0 = np.array([self._cov.scale])
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=5)

        def func(x, i):
            self._cov.length = x[i]
            return self._cov.K_grad_i(1)

        def grad(x, i):
            self._cov.length = x[i]
            return self._cov.K_hess_i_j(1, 1)

        x0 = np.array([self._cov.scale])
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=5)


    def test_input(self):
        with self.assertRaises(ValueError):
            SQExpCov(np.array([[np.inf]]))

        with self.assertRaises(ValueError):
            SQExpCov(np.array([[np.nan]]))

        with self.assertRaises(NotArrayConvertibleError):
            SQExpCov("Ola meu querido.")

if __name__ == '__main__':
    unittest.main()



import numpy.linalg as nla
import scipy as sp

def svd_reduce(X, tol = 1e-9):
    U, Sh, V = nla.svd(X, full_matrices=0)
    I = Sh < tol
    if I.any():
        #warnings.warn('G has dependent columns, dimensionality reduced')
        Sh = Sh[~I]
        U  = U[:, ~I]
        V  = sp.eye(Sh.shape[0])
        X  = U * Sh[sp.newaxis,:]
    S = Sh**2
    return X, U, S, V



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import limix
import limix.deprecated as dlimix
from limix.test import data
import os
import sys


class CLMM_test(unittest.TestCase):
    """test class for CLMM"""

    def setUp(self):
        self.datasets = ['lmm_data1']
        self.dir_name = os.path.dirname(os.path.realpath(__file__))

    def test_lmm1(self):
        """basic test, comapring pv"""
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            lmm = dlimix.CLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            lmm.process()
            pv = lmm.getPv().ravel()
            BetaSte = lmm.getBetaSNPste().ravel()
            Beta = lmm.getBetaSNP()
            D2pv= ((SP.log10(pv)-SP.log10(D['pv']))**2)
            # D2Beta= (Beta-D['Beta'])**2
            # D2BetaSte = (BetaSte-D['BetaSte'])**2
            RV = SP.sqrt(D2pv.mean())<1E-6
            # RV = RV & (D2Beta.mean()<1E-6)
            # RV = RV & (D2BetaSte.mean()<1E-6)
            self.assertTrue(RV)

    def test_exceptions(self):
        D = data.load(os.path.join(self.dir_name,self.datasets[0]))
        lmm  = dlimix.CLMM()
        N = 100
        K = SP.eye(N)
        X = SP.randn(N,100)
        Y = SP.randn(N+1,1)
        Cov = SP.randn(N,1)
        lmm.setK(K)
        lmm.setSNPs(X)
        lmm.setCovs(Cov)
        lmm.setPheno(Y)
        try:
            lmm.process()
        except Exception,e:
            self.assertTrue(1==1)
            pass

    def test_permutation(self):
        #test permutation function
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            perm = SP.random.permutation(D['X'].shape[0])
            #1. set permuattion
            lmm = dlimix.CLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            if 1:
                #pdb.set_trace()
                perm = SP.array(perm,dtype='int32')#Windows needs int32 as long -> fix interface to accept int64 types
            lmm.setPermutation(perm)
            lmm.process()
            pv_perm1 = lmm.getPv().ravel()
            #2. do by hand
            lmm = dlimix.CLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'][perm])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            lmm.process()
            pv_perm2 = lmm.getPv().ravel()
            D2 = (SP.log10(pv_perm1)-SP.log10(pv_perm2))**2
            RV = SP.sqrt(D2.mean())
            self.assertTrue(RV<1E-6)



class CInteractLMM_test:
    """Interaction test"""
    def __init__(self):
        pass

    def test_all(self):
        RV = False
        #print 'CInteractLMM IMPLEMENTED %s' % message(RV)


if __name__ == '__main__':
    unittest.main()



import sys
sys.path.append('./..')
import h5py
import pdb
import scipy as SP
import scipy.stats as ST
import scipy.linalg as LA
import time as TIME
import copy
import warnings
import os


class Splitter():

    def __init__(self,pos=None,chrom=None,separate_chroms=False):
        """
        Constructor
        Args:
            pos:        position
            chrom:      chromosome
        """
        # assert
        assert pos is not None, 'Slider:: set pos'
        assert chrom is not None, 'Slider:: set chrom'

        self.pos   = pos
        self.chrom = chrom
        # sep chroms
        self.separate_chroms = separate_chroms
        # windows
        self.windows = None
        # additional info for windows
        self.info   = {}
        pass

    def splitGeno(self,method='slidingWindow',size=5e4,step=None,annotation_file=None,cis=1e4,funct=None,minSnps=1.,maxSnps=SP.inf,cache=False,out_dir='./cache',fname=None,rewrite=False):
        """
        split into windows
        Args:
            method:     method used to slit the windows:
                        'slidingWindow':    uses a sliding window
                        'geneWindow':       uses windows centered on genes
            size:       window size used in slidingWindow method
            step:       moving step used in slidingWindow method
            annotation_file:    file containing the annotation file for geneWindow method
            minSnps:    only windows with nSnps>=minSnps are considered
            maxSnps:    only windows with nSnps>=maxSnps are considered
            cache:      if results need to be cashed
            out_dir:    folder used for caching
            fname:      name of the hdf5 file used for caching
            rewrite:    if true, rewrite the existing cache hdf5 file
        """
        self.info= {} # forget everything
        # check if it is necessary to read form file or not
        read_from_file = False
        if cache:
            assert fname is not None, 'Splitter:: specify fname'
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            out_file = os.path.join(out_dir,fname)
            read_from_file = os.path.exists(out_file) and not rewrite

        self.size = size

        if read_from_file:
            # reads from file
            f = h5py.File(out_file,'r')
            self._wnd_pos = f['wnd_pos'][:]
            self._idx_wnd_start = f['idx_wnd_start'][:]
            self._nSnps = f['nSnps'][:]
            f.close()
        else:
            # calculates windows using the indicated method
            if method=='slidingWindow':
                self._splitGenoSlidingWindow(size=size,step=step,minSnps=minSnps,maxSnps=maxSnps)
            elif method=='geneWindow':
                self._splitGenoGeneWindow(annotation_file=annotation_file,cis=cis,funct=funct,minSnps=minSnps,maxSnps=maxSnps)

            if cache:
                f = h5py.File(out_file,'w')
                f.create_dataset('wnd_pos', data=self._wnd_pos)
                f.create_dataset('idx_wnd_start', data=self._idx_wnd_start)
                f.create_dataset('nSnps', data=self._nSnps)
                f.close()

    def _splitGenoSlidingWindow(self,size=5e4,step=None,minSnps=1.,maxSnps=SP.inf):
        """
        split into windows using a slide criterion
        Args:
            size:       window size
            step:       moving step (default: 0.5*size)
            minSnps:    only windows with nSnps>=minSnps are considered
            maxSnps:    only windows with nSnps>=maxSnps are considered
        """
        if step is None:    step = 0.5*size
        chroms  = SP.unique(self.chrom)
        wnd_pos       = []
        idx_wnd_start = []
        nSnps         = []
        wnd_i = 0

        nSnps = []
        for chrom_i in chroms:
            start = 0
            Ichrom = self.chrom==chrom_i
            idx_chrom_start = SP.where(Ichrom)[0][0]
            pos_chr = self.pos[Ichrom]
            pos_chr_max = pos_chr.max()
            while 1:
                if start>pos_chr_max: break
                end = start+size
                Ir = (self.pos>=start)*(self.pos<end)
                _nSnps = Ir.sum()
                if _nSnps>minSnps and _nSnps<maxSnps:
                    wnd_pos.append([chrom_i,start,start+size])
                    nSnps.append(_nSnps)
                    idx_wnd_start.append(idx_chrom_start+SP.where(Ir)[0][0])
                    wnd_i+=1
                start += step
        self._wnd_pos = SP.array(wnd_pos)
        self._idx_wnd_start = SP.array(idx_wnd_start)
        self._nSnps = SP.array(nSnps)

    def _splitGenoGeneWindow(self,annotation_file=None,cis=1e4,funct='protein_coding',minSnps=1.,maxSnps=SP.inf):
        """
        split into windows based on genes
        """
         #1. load annotation
        assert annotation_file is not None, 'Splitter:: specify annotation file'
        try:
            f = h5py.File(annotation_file,'r')
            geneID = f['geneID'][:]
            gene_chrom = f['chrom'][:]
            gene_start = f['start'][:]
            gene_end = f['end'][:]
            gene_strand = f['strand'][:]
            gene_function = f['function'][:]
            f.close()
        except:
            print 'Splitter:: format annotation file not valid'

        # if funct is not None, it has to be a list
        if funct is not None and funct!=list:   funct=[funct]

        windows = []
        nSnps   = []
        Igene   = []
        #2. calculates windows 
        for gene_i in range(geneID.shape[0]):
            if funct is not None:
                if gene_function[gene_i] not in funct:
                    Igene.append(False)
                    continue
            wnd = [gene_chrom[gene_i],gene_start[gene_i]-cis,gene_end[gene_i]+cis]
            Ir = (self.chrom==wnd[0])*(self.pos>=wnd[1])*(self.pos<=wnd[2])
            _nSnps = Ir.sum()
            if _nSnps>=minSnps and _nSnps<=maxSnps:
                windows.append(wnd)
                nSnps.append(_nSnps)
                Igene.append(True)
            else:
                Igene.append(False)
        Igene = SP.array(Igene)
        self.info['nSnps'] = SP.array(nSnps)
        self.info['geneID']        = geneID[Igene]
        self.info['gene_start']    = gene_start[Igene]
        self.info['gene_end']      = gene_end[Igene]
        self.info['gene_strand']   = gene_strand[Igene]
        self.info['gene_function'] = gene_function[Igene]
        return SP.array(windows)

    @property
    def wnd_pos(self):
        return self._wnd_pos

    @property
    def idx_wnd_start(self):
        return self._idx_wnd_start

    @property
    def nSnps(self):
        return self._nSnps

    @property
    def nWindows(self):
        return self.wnd_pos.shape[0]




"""LMM testing code"""
import unittest
import scipy as SP
import numpy as np
import sys
from limix.core.covar import FixedCov
from limix.utils.check_grad import mcheck_grad
from limix.core.type.exception import NotArrayConvertibleError

class TestFixed(unittest.TestCase):
    def setUp(self):
        pass

    def test_input(self):
        FixedCov(np.eye(10))
        with self.assertRaises(NotArrayConvertibleError):
            FixedCov("Ola meu querido.")

        A = np.asarray([[np.inf], [1.]], dtype=float)
        with self.assertRaises(ValueError):
            FixedCov(A)

if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.

"""
PANAMA module in limix
"""
import limix.deprecated.modules.qtl as qtl
import limix.deprecated.stats.fdr as fdr
from limix.deprecated.stats.pca import *
import limix.deprecated as dlimix
import scipy as sp
import scipy.linalg as la
import pdb
import scipy.linalg as linalg
import time

class varDecompSet:
    """variance decomposition set class"""
    def __init__(self,Y=None,Ks=None,standardize=True,noise=None):
        """
        Args:
            X: alternatively SNP data
            Y: alternativel expression data
            Ks: list of covariance matrices
            standardize: if True, phenotypes are standardized
        """
        assert Y is not None, 'Specify Y'
        assert Ks is not None, 'Specify Ks'
        if type(Ks)!=list: Ks = [Ks]
        self.Y  = Y.copy()
        self.Ks = Ks
        if standardize:
            self.Y -= self.Y.mean(axis=0)
            self.Y /= self.Y.std(axis=0)
        self.N = self.Y.shape[0]
        self.P = self.Y.shape[1]
        self.noise = noise
        self.n_terms = len(self.Ks)+1
        if self.noise is 'off':
            self.n_terms-=1

    def train(self,jitter=1e-4):
        """train vds module"""
        covar  = limix.CSumCF()
        covar_params = []
        for K in self.Ks:
            covar.addCovariance(limix.CFixedCF(K+jitter*sp.eye(self.N)))
            covar_params.append(sp.ones(1)/sp.sqrt(self.n_terms))
        if self.noise is 'correlated':
            _cov1 = limix.CFixedCF(sp.eye(int(self.Y.shape[0]/2)))
            self.cov_noise = limix.CFixedDiagonalCF(limix.CFreeFormCF(2),sp.ones(2))
            self.cov_noise.setParamMask(sp.array([0,1,0]))
            self.Knoise = limix.CKroneckerCF(_cov1,self.cov_noise)
            covar.addCovariance(self.Knoise)
            covar_params.append(sp.ones(1)/sp.sqrt(self.n_terms))
            covar_params.append(sp.array([1.,1e-3,1.]))
        elif self.noise is not 'off':
            covar.addCovariance(limix.CFixedCF(sp.eye(self.N)))
            covar_params.append(sp.ones(1)/sp.sqrt(self.n_terms))

        hyperparams = limix.CGPHyperParams()
        covar_params = sp.concatenate(covar_params)
        hyperparams['covar'] = covar_params
        constrainU = limix.CGPHyperParams()
        constrainL = limix.CGPHyperParams()
        constrainU['covar'] = +5*sp.ones_like(covar_params);
        constrainL['covar'] = -5*sp.ones_like(covar_params);

        self.gp=limix.CGPbase(covar,limix.CLikNormalNULL())
        self.gp.setY(self.Y)
        lml0 = self.gp.LML(hyperparams)
        dlml0 = self.gp.LMLgrad(hyperparams)
        gpopt = limix.CGPopt(self.gp)
        gpopt.setOptBoundLower(constrainL);
        gpopt.setOptBoundUpper(constrainU);

        t1 = time.time()
        conv = gpopt.opt()
        t2 = time.time()

        RV = {'Converged': True, 'time': t2-t1}
        return RV

    def getVarianceComps(self):
        """
        Returns:
            vector of variance components of the PANAMA, Kpop and noise contributions
        """
        if self.noise is 'correlated':
            RV = self.gp.getParams()['covar'][:self.n_terms-1]**2
        else:
            RV = self.gp.getParams()['covar']**2
        return RV

    def getNoise(self):
        assert self.noise is 'correlated', 'work only if noise is \'correlated\'!'
        return self.gp.getParams()['covar'][self.n_terms-1]**2*self.cov_noise.K()



from covar_base import Covariance
import pdb
import numpy as np
import scipy as sp
from limix.core.type.cached import Cached, cached

class KronCov(Covariance):
    """
    Kronecker product between two covariances
    """

    def __init__(self, C, R, Iok=None):
        """
        Args:
            C:     column LIMIX covariance 
            R:     row numpy covariance matrix 
        """
        Covariance.__init__(self)
        self._C = C
        self._R = R
        self.dim = C.dim * R.shape[0]
        self.Iok = Iok 
        C.register(self.clear_all)

    #####################
    # Properties
    #####################
    @property
    def C(self):
        return self._C

    @property
    def R(self):
        return self._R

    @property
    def Iok(self):
        return self._Iok

    #####################
    # Setters
    #####################
    @Iok.setter
    def Iok(self, value):
        if value is not None:
            assert len(value.shape)==1, 'must be a 1-dimensioanal array'
            assert value.shape[0]==self.dim, 'Dimension mismatch'
            self.dim = value.sum()
        self._Iok = value

    #####################
    # Params handling
    #####################
    def setParams(self,params):
        self.C.setParams(params)

    def getParams(self):
        return self.C.getParams()

    def getNumberParams(self):
        return self.C.getNumberParams() 

    ####################
    # Predictions
    ####################
    @property
    def use_to_predict(self):
        return self.C.use_to_predict

    @use_to_predict.setter
    def use_to_predict(self,value):
        self.C.use_to_predict = value

    #####################
    # Cached
    #####################
    @cached('covar_base')
    def K(self):
        R = sp.kron(self.C.K(), self.R)
        if self.Iok is not None:
            R = R[self.Iok][:, self.Iok]
        return R

    @cached('covar_base')
    def Kcross(self):
        #TODO: create support for Rcross
        # it woud be simply:
        #return sp.kron(self.C.K(), self.Rcross)
        pass

    @cached('covar_base')
    def K_grad_i(self,i):
        R = sp.kron(self.C.K_grad_i(i), self.R)
        if self.Iok is not None:
            R = R[self.Iok][:, self.Iok]
        return R

    @cached('covar_base')
    def K_hess_i_j(self, i, j):
        R = sp.kron(self.C.K_hess_i_j(i, j), self.R)
        if self.Iok is not None:
            R = R[self.Iok][:, self.Iok]
        return R

    def _calcNumberParams(self):
        self.n_params = self.C.getNumberParams() 
        return self.n_params

    ####################
    # Interpretable Params
    ####################
    def getInterParams(self):
        return self.C.getInterParams()

    def K_grad_interParam_i(self,i):
        R = sp.kron(self.C.K_grad_interParam_i(i), self.R)
        if self.Iok is not None:
            R = R[self.Iok][:,self.Iok]
        return R

    def setFIinv(self, value):
        self.C.setFIinv(value)

if __name__=='__main__':
    from limix.core.covar import FreeFormCov
    from limix.utils.preprocess import covar_rescale

    # define row caoriance
    dim_r = 10
    X = sp.rand(dim_r, dim_r)
    R = covar_rescale(sp.dot(X,X.T))

    # define col covariance
    dim_c = 3
    C = FreeFormCov(dim_c)

    cov = KronCov(C, R) 
    cov.setRandomParams()
    print cov.K()
    print cov.K_grad_i(0)




'''
Created on Dec 19, 2013

@author: johannes stephan, barbara rakitsch
'''

from sklearn.linear_model import Lasso
from sklearn.metrics import mean_squared_error
import sklearn.cross_validation as cross_validation

from varianceDecomposition import VarianceDecomposition
import qtl
import scipy as SP
import numpy as NP
import scipy.stats as ST
import time
import scipy.linalg as LA
import scipy.optimize as OPT
import pdb


def compute_linear_kernel(X,idx=None,jitter=1e-3,standardize=True):
    """
    compute linear kernel

    X           : SNP data [N x F]
    idx         : boolean vector of size F, indicating which SNPs to use to build the covariance matrix
    standardize : if True (default), covariance matrix is standardized to unit variance
    jitter      : adds jitter to the diagonal of the covariance matrix (default: 1e-3)
    """
    N = X.shape[0]
    if idx is not None:
        K = SP.dot(X[:,idx],X[:,idx].T)
    else:
        K = SP.dot(X,X.T)

    if standardize:
        K /= SP.diag(K).mean()

    if jitter:
        K += jitter*SP.eye(N)

    return K


def runStabilitySelection(estimator,X,y,K=None,n_repeats=100,frac_sub=0.5,verbose=False):
    """
    estimator   : (Lmm-)Lasso model
    X           : SNP data
    y           : phenotype vector
    K           : background covariance matrix
    n_repeats   : number of repetitions
    frac_sub    : size of the subsample
    """
    N,F = X.shape
    Nsub = int(frac_sub*N)

    W_nonzero = SP.zeros((n_repeats,F),dtype=bool)

    for irep in range(n_repeats):
        if verbose: print 'Running %d/%d'%(irep,n_repeats)
        iperm = SP.random.permutation(N)[:Nsub]

        if K is not None:
            estimator.fit(X[iperm],y[iperm],K=K[iperm][:,iperm])
        else:
            estimator.fit(X[iperm],y[iperm])

        W_nonzero[irep] = estimator.coef_!=0

    p_stab = W_nonzero.mean(0)
    return p_stab


def testLassoBackgroundModel(estimator,Xfg,Xbg,y,use_Kgeno=True,**kwargs):
    """
    run a linear mixed model on the foreground SNPs (Xfg), while estimating SNPs for the background covariance matrix via the Lasso usting the background SNPs(Xbg).

    estimator   : (Lmm-)Lasso model
    Xbg         : SNP data for the background model
    Xfg         : SNP data for the foreground model
    y           : phenotype vector
    use_Kgene   : if True (default), background covariance matrix is estimated and used.
    """
    if use_Kgeno:
        Kbg = compute_linear_kernel(Xbg)
        estimator.fit(Xfg,y,Kbg)
        iactive = estimator.coef_!=0

        if iactive.any():
            Kfg = compute_linear_kernel(Xbg,iactive)
            vd = VarianceDecomposition(y)
            vd.addRandomEffect(is_noise=True)
            vd.addRandomEffect(Kbg)
            vd.addRandomEffect(Kfg)
            vd.optimize()
            K = vd.gp.getCovar().K()
        else:
            K = Kbg

    else:
        estimator.fit(Xbg,y)
        idx = estimator.coef_!=0
        K   = compute_linear_kernel(Xbg,idx)

    lmm = qtl.test_lmm(Xfg,y,K=K,**kwargs)
    pv  = lmm.getPv()

    return pv

def runCrossValidation(estimator,X,y,alphas,K=None,n_folds=10,verbose=False):
    """ run cross-validation

    estimator   : (Lmm-)Lasso model
    X           : SNP data
    y           : phenotype vector
    alphas      : list of l1-regularization parameter
    K           : background covariance matrix
    n_folds     : number of folds used in cross-validation
    """

    """ setting up """
    N = X.shape[0]
    kfold = cross_validation.KFold(N, n_folds=n_folds)
    n_alphas = len(alphas)
    MSE_train = SP.zeros((n_folds,n_alphas))
    MSE_test  = SP.zeros((n_folds,n_alphas))
    W_nonzero = SP.zeros((n_folds,n_alphas))

    t1 = time.time()
    ifold = 0
    for train,test in kfold:
        if verbose: print 'running fold %d'%ifold

        """ splitting into training and testing """
        X_train = X[train]
        X_test  = X[test]
        y_train = y[train]
        y_test  = y[test]
        if K is not None:
            K_train = K[train][:,train]
            K_test  = K[test][:,train]

        for ialpha in range(n_alphas):
            estimator.set_params(alpha=alphas[ialpha])
            if K is None:
                estimator.fit(X_train,y_train)
                ytrain_star = estimator.predict(X_train)
                ytest_star  = estimator.predict(X_test)
            else:
                estimator.fit(X_train,y_train,K_train)
                ytrain_star = estimator.predict(X_train,K_train)
                ytest_star  = estimator.predict(X_test, K_test)

            MSE_train[ifold,ialpha]  = mean_squared_error(ytrain_star,y_train)
            MSE_test[ifold,ialpha]   = mean_squared_error(ytest_star,y_test)
            W_nonzero[ifold,ialpha]  = SP.sum(estimator.coef_!=0)

        ifold +=1

    t2 = time.time()
    if verbose: print 'finished in %.2f seconds'%(t2-t1)
    return MSE_train,MSE_test,W_nonzero




class LmmLasso(Lasso):
    """
    Lmm-Lasso classo
    """
    def __init__(self, alpha=1., **lasso_args):
        """
        Extension to the sklearn's LASSO to model population structure

        alpha: l1-regularization parameter
        """
        super(LmmLasso, self).__init__(alpha=alpha, **lasso_args)
        self.msg = 'lmmlasso'




    def fit(self, X, y, K, standardize=False, verbose=False,**lasso_args):
        """
        fitting the model

        X: SNP data
        y: phenotype data
        K: backgroundcovariance matrix
        standardize: if True, genotypes and phenotypes are standardized
        """

        if y.ndim == 2:
            assert y.shape[1]==1, 'Only one phenotype can be processed at at time.'
            y = y.flatten()
        time_start = time.time()
        [n_s, n_f] = X.shape
        assert X.shape[0] == y.shape[0], 'dimensions do not match'
        assert K.shape[0] == K.shape[1], 'dimensions do not match'
        assert K.shape[0] == X.shape[0], 'dimensions do not match'

        """ standardizing genotypes and phenotypes """
        if standardize:
            X -= X.mean(axis=0)
            X /= X.std(axis=0)
            y -= y.mean(axis=0)
            y /= y.std(axis=0)

        """ training null model """
        vd = VarianceDecomposition(y)
        vd.addRandomEffect(is_noise=True)
        vd.addRandomEffect(K)
        vd.optimize()
        varComps = vd.getVarianceComps()
        delta0   = varComps[0,0]/varComps.sum()
        self.varComps = varComps

        S,U = LA.eigh(K)

        """ rotating data """
        Sdi = 1. / (S + delta0)
        Sdi_sqrt = SP.sqrt(Sdi)
        SUX = SP.dot(U.T, X)
        SUX = SUX * SP.tile(Sdi_sqrt, (n_f, 1)).T
        SUy = SP.dot(U.T, y)
        SUy = Sdi_sqrt * SUy

        """ fitting lasso """
        super(LmmLasso, self).fit(SUX, SUy, **lasso_args)
        yhat = super(LmmLasso, self).predict(X)
        self.w_ridge = LA.solve(K + delta0 * SP.eye(n_s), y - yhat)

        time_end = time.time()
        time_diff = time_end - time_start
        if verbose: print '... finished in %.2fs'%(time_diff)

        return self

    def getVarianceComps(self,univariance=False):
        """
        Return the estimated variance components

        Args:
            univariance:   Boolean indicator, if True variance components are normalized to sum up to 1 for each trait
        Returns:
            variance components of all random effects [noise, signal]
        """
        if univariance:
            self.varComps /= self.varComps.mean()

        return self.varComps

    def predict(self, Xstar, Kstar=None):
        """
        predicting the phenotype

        Xstar: SNP data
        Kstar: covariance matrix between test and training samples
        """

        assert self.w_ridge.shape[0]==Kstar.shape[1], 'number of training samples is not consistent.'
        assert self.coef_.shape[0]==Xstar.shape[1],   'number of SNPs is not consistent.'
        assert Xstar.shape[0]==Kstar.shape[0],   'number of test samples is not consistent.'

        fixed_effect = super(LmmLasso, self).predict(Xstar)
        if Kstar is not None:
            return fixed_effect + SP.dot(Kstar, self.w_ridge)
        else:
            return fixed_effect



import scipy.linalg as la
import numpy as np

class psd_solver(object):
    """a linear equation solver for symmetric positive semi-definite matrices"""

    def __init__(self, A, lower=True, threshold=1e-10,check_finite=True,overwrite_a=False):
        self._s=None
        self._U=None
        self._chol=None
        self._lower=lower
        try:
            self._chol=la.cholesky(A,overwrite_a=overwrite_a,check_finite=True).T
    
        except la.LinAlgError:
            s,U = la.eigh(A,lower=lower)
            i_pos = (s>threshold)
            if i_pos.any():
                self._s = s[i_pos]
                self._U = U[:,i_pos]
            

            

            
    def solve(self,b,overwrite_b=False,check_finite=True):
        """
        solve A \ b
        """
        if self._s is not None:
            res = self._U.T.dot(b)
            res /= self._s[:,np.newaxis]
            res = self._U.dot(res)
        elif self._chol is not None:
            res = la.cho_solve((self._chol,self._lower),b=b,overwrite_b=overwrite_b,check_finite=check_finite)
        else:
            res = np.zeros(b.shape)
        return res

class psd_solver_any(object):
    """a linear equation solver for symmetric positive semi-definite matrices for the case where only any effects are present"""

    def __init__(self, A, lower=True, threshold=1e-10,check_finite=True,overwrite_a=False):
        self.solver = []
        (self.P,self.dof_any) = A.shape[0:2]
        self._lower=lower
        #This is trivially parallelizable:
        for p in xrange(A.shape[0]):
            self.solver.append(psd_solver(A[p]))

    def solve(self,b,overwrite_b=False,check_finite=True, p=None):
        """
        solve A \ b
        """
        if p is None:
            assert b.shape[:2]==(len(self.solver),self.dof_any)
            solution = np.empty(b.shape)
            #This is trivially parallelizable:
            for p in xrange(self.P):
                solution[p] = self.solver[p].solve(b=b[p])
            return solution
        else:
            return self.solver[p].solve(b=b)

class PsdSolverKron(object):
    """
    efficient general Kronecker solver for the case where any effects are present
    """

    def __init__(self, A_any, A, AA_any, lower=True, threshold=1e-10, check_finite=True, overwrite_a=False):
        self._lower=lower
        self.schur_solver = None
        self.DinvC = None
        self.A_any_solver = None

        if (A_any is not None) and A_any.shape[0]:#any effects are present
            self.A_any_solver = psd_solver_any(A=A_any, lower=lower, threshold=threshold,check_finite=check_finite,overwrite_a=overwrite_a)
            if (A is not None) and A.shape[0]>0:
                self.DinvC = self.A_any_solver.solve(AA_any)
                if overwrite_a:
                    schur = A
                    schur -= np.tensordot(AA_any,self.DinvC,axes=([0,1],[0,1]))
                else:
                    schur = A - np.tensordot(AA_any,self.DinvC,axes=([0,1],[0,1]))
                self.schur_solver = psd_solver(A=schur, lower=lower, threshold=threshold,check_finite=check_finite,overwrite_a=overwrite_a)
            
        elif (A is not None) and A.shape[0]>0:
            self.schur_solver = psd_solver(A=A, lower=lower, threshold=threshold,check_finite=check_finite,overwrite_a=overwrite_a)
          

    def solve(self, b_any, b, check_finite=True, p=None):
        """
        solve A \ b
        """
        #assert b.shape[:2]==(len(self.solver),self.dof_any)
        

        if self.schur_solver is None and self.A_any_solver is None:
            assert ( (b is None) or (b.shape[0]==0) ) and ( (b_any is None) or (b_any.shape[0]==0) ), "shape missmatch"
            return b, b_any
        elif self.schur_solver is None:
            assert (b is None) or (b.shape[0]==0), "shape missmatch"
            solution_any = self.A_any_solver.solve(b=b_any,p=p)
            return b,solution_any
        elif self.A_any_solver is None:
            assert (b_any is None) or (b_any.shape[0]==0), "shape missmatch"
            solution = self.schur_solver.solve(b=b, check_finite=check_finite)
            return solution, b_any
        else:
            assert p is None, "p is not None"
            cross_term = np.tensordot(self.DinvC,b_any,axes=([0,1],[0,1]))
            solution = self.schur_solver.solve(b=(b - cross_term), check_finite=check_finite)
            solution_any = self.A_any_solver.solve(b=b_any, check_finite=check_finite, p=p)
            solution_any -= self.DinvC.dot(solution)
            return solution, solution_any



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
PCA related utility function
"""
import scipy as sp
import numpy as np 
import pdb
import numpy.linalg as linalg
import scipy as sp

def PCA(Y, components):
	"""
	run PCA, retrieving the first (components) principle components
	return [s0, eig, w0]
	s0: factors
	w0: weights
	"""

	N,D = Y.shape
	sv = linalg.svd(Y, full_matrices=0);
	[s0, w0] = [sv[0][:, 0:components], np.dot(np.diag(sv[1]), sv[2]).T[:, 0:components]]
	v = s0.std(axis=0)
	s0 /= v;
	w0 *= v;
	return [s0, w0]

	if N>D:
		sv = linalg.svd(Y, full_matrices=0);
		[s0, w0] = [sv[0][:, 0:components], np.dot(np.diag(sv[1]), sv[2]).T[:, 0:components]]
		v = s0.std(axis=0)
		s0 /= v;
		w0 *= v;
		return [s0, w0]
	else:
		K=np.cov(Y)
		sv = linalg.eigh(K)
		std_var = np.sqrt(sv[0])
		pc = sv[1]*std_var[np.newaxis(),0]
		#import ipdb
		#ipdb.set_trace()
		return [pc,std_var]

def PC_varExplained(Y,standardized=True):
    """
    Run PCA and calculate the cumulative fraction of variance
    Args:
        Y: phenotype values
        standardize: if True, phenotypes are standardized
    Returns:
        var: cumulative distribution of variance explained
    """
    # figuring out the number of latent factors
    if standardized:
        Y-=Y.mean(0)
        Y/=Y.std(0)
    covY = sp.cov(Y)
    S,U = linalg.eigh(covY+1e-6*sp.eye(covY.shape[0]))
    S = S[::-1]
    rv = np.array([S[0:i].sum() for i in range(1,S.shape[0])])
    rv/= S.sum()
    return rv

if __name__ == '__main__':
	Y = np.random.randn(10,20)
	components = 5
	#import pca
	pca = PCA(Y, components)
	pass



import unittest
import numpy as np
from limix.core.covar import FreeFormCov 
from limix.core.covar import KronCov
from limix.utils.preprocess import covar_rescale
from limix.utils.check_grad import mcheck_grad
import scipy as sp

class TestKronCov(unittest.TestCase):
    def setUp(self):
        np.random.seed(1)
        dim_r = 10
        dim_c = 3
        X = sp.rand(dim_r, dim_r)
        R = covar_rescale(sp.dot(X,X.T))
        C = FreeFormCov(dim_c)
        self._cov = KronCov(C, R)
        self._Iok = sp.randn(self._cov.dim)<0.9

    def test_Kgrad(self):

        cov = self._cov

        def func(x, i):
            cov.setParams(x)
            return cov.K()

        def grad(x, i):
            cov.setParams(x)
            return cov.K_grad_i(i)

        x0 = cov.getParams()
        err = mcheck_grad(func, grad, x0)

        np.testing.assert_almost_equal(err, 0., decimal=5)

    def test_softKronKgrad(self):

        cov = self._cov
        cov.Iok = self._Iok

        def func(x, i):
            cov.setParams(x)
            return cov.K()

        def grad(x, i):
            cov.setParams(x)
            return cov.K_grad_i(i)

        x0 = cov.getParams()
        err = mcheck_grad(func, grad, x0)

        np.testing.assert_almost_equal(err, 0., decimal=5)

    #def test_Khess(self):
    #
    #    cov = self._cov

    #    for j in range(cov.getNumberParams()):

    #        def func(x, i):
    #            cov.setParams(x)
    #            return cov.K_grad_i(j)

    #        def grad(x, i):
    #            cov.setParams(x)
    #            return cov.K_hess_i_j(j, i)

    #        x0 = cov.getParams()
    #        err = mcheck_grad(func, grad, x0)
    #        np.testing.assert_almost_equal(err, 0.)

if __name__ == '__main__':
    unittest.main()



# USAGE: Please, look at the end of the file for an example.

class Observed(object):
    def __init__(self):
        self._init_interesteds()

    def _init_interesteds(self):
        self._interesteds = dict()

    def register(self, interested, event='default'):
        if not hasattr(self, '_interesteds'):
            self._init_interesteds()
        if event not in self._interesteds.keys():
            self._interesteds[event] = []
        self._interesteds[event].append(interested)

    def _notify(self, event='default'):
        if not hasattr(self, '_interesteds'):
            self._init_interesteds()
        if event in self._interesteds:
            for i in self._interesteds[event]:
                i()

if __name__ == '__main__':
    class A(Observed):
        def set_params(self):
            self._notify()

        def trigger_event1(self):
            self._notify('event1')

    class B(object):
        def __init__(self, a):
            a.register(self.call_me)
            a.register(self.call_me_on_event1, 'event1')

        def call_me(self):
            print 'call_me'

        def call_me_on_event1(self):
            print 'call_me_on_event1'

    a = A()
    b = B(a)
    a.set_params() # prints call_me
    a.set_params() # prints call_me
    a.trigger_event1() # prints call_me_on_event1



#! /usr/bin/env python
# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
# All rights reserved.
#
# LIMIX is provided under a 2-clause BSD license.
# See license.txt for the complete license.

import time
import h5py
import pdb
from limix.deprecated.io.conversion import *




def entry_point():
    infostring = "limix_conveter.py, Copyright(c) 2014, The LIMIX developers\nlast modified: %s" % time.ctime(os.path.getmtime(__file__))
    print (infostring)

    runner = LIMIX_converter(infostring=infostring)
    (options,args) = runner.parse_args()
    runner.run()



#!/usr/bin/env python

import nlopt
import numpy as np

print ('nlopt version='+nlopt.__version__)

def f(x, grad):
    F=x[0]
    L=x[1]
    E=x[2]
    I=x[3]
    D=F*L**3/(3.*E*I)
    return D

n = 4
opt = nlopt.opt(nlopt.LN_COBYLA, n)
opt.set_min_objective(f)
lb = np.array([40., 50., 30e3, 1.])
ub = np.array([60., 60., 40e3, 10.])
x = (lb+ub)/2.
opt.set_lower_bounds(lb)
opt.set_upper_bounds(ub)
opt.set_xtol_rel(1e-3)
opt.set_ftol_rel(1e-3)
xopt = opt.optimize(x)

opt_val = opt.last_optimum_value()
result = opt.last_optimize_result()
print ('opt_result='+str(result))
print ('optimizer='+str(xopt))
print ('opt_val='+str(opt_val))



'''
Created on Jan 24, 2012

@author: johannes
'''
import sys
import numpy as NP
import scipy as SP
import lmm_fast
import limix
from limix.ensemble import SplittingCore as CSP

def is_leaf(node_ind, child_nodes):
    return child_nodes[node_ind] == 0

def get_ancestors(node_ind, node, parents):
    ancestors = SP.empty(SP.int_(SP.floor(SP.log2(node+1))), dtype='int')
    for i in SP.arange(ancestors.size):
        node_ind = parents[node_ind]
        ancestors[i] = node_ind
    return ancestors

def get_covariates(node_ind, node, parents, sample, start_index, end_index):
    node_indexes = get_ancestors(node_ind,node, parents)
    Covariates = SP.zeros((sample.size,node_indexes.size+1), order='F')
    #node itself

    Covariates[sample[start_index[node_ind]:end_index[node_ind]],0] = 1
    j = 1
    for node_ind in node_indexes:
        Covariates[sample[start_index[node_ind]:end_index[node_ind]],j] = 1
        j += 1
    return Covariates

def checkMaf(X, maf):
    Xmaf = (X==1).sum(axis=0)
    Iok = (Xmaf>=(maf*X.shape[0]))
    return SP.where(Iok)[0]

def scale_K(K, verbose=False):
    """scale covariance K such that it explains unit variance"""
    c = SP.sum((SP.eye(len(K)) - (1.0 / len(K)) * SP.ones(K.shape)) * SP.array(K))
    scalar = (len(K) - 1) / c
    if verbose:
        print 'Kinship scaled by: %0.4f' % scalar
    K = scalar * K
    return K

def estimateKernel(X, maf):
    #1. maf filter
    Xpop = SP.float_(X[:,checkMaf(X, maf)]).copy()
    Xpop -= Xpop.mean(axis=0)
    Xpop /= Xpop.std(axis=0)
    Kpop  =  SP.dot(Xpop,Xpop.T)
    return scale_K(Kpop)

def estimate_bias(Uy, U, S, ldelta):
    UC = SP.dot(U.T,SP.ones_like(Uy))
    _, beta, _ = lmm_fast.nLLeval(ldelta, Uy[:,0], UC, S, MLparams=True)
    return beta[0]

def check_predictors(X, noderange, rmind):
    Xout = X[SP.ix_(noderange, rmind)]
    X_sum = SP.sum(Xout,0)
    indexes = (X_sum != Xout.shape[0]) & (X_sum != 0)
    return rmind[indexes]

def cpp_best_split_full_model(X, Uy, C, S, U, noderange, delta,
                              save_memory=False):
    """wrappe calling cpp splitting function"""
    return CSP.best_split_full_model(X, Uy, C, S, U, noderange, delta)

def best_split_full_model(X,
                          Uy,
                          C,
                          S,
                          U,
                          noderange,
                          delta):
    mBest = -1
    sBest = -float('inf')
    score_best = -float('inf')
    left_mean = None
    right_mean = None
    ldelta = SP.log(delta)
    levels = map(SP.unique, X[noderange].T)
    feature_map = []
    s = []
    UXt = []
    cnt = 0
    for i in xrange(X.shape[1]):
        lev = levels[i]
        for j in xrange(lev.size-1):
            split_point = SP.median(lev[j:j+2])
            x = SP.int_(X[noderange,i] > split_point)
            UXt.append(SP.dot(U.T[:,noderange], x))
            feature_map.append(i)
            s.append(split_point)
            cnt += 1
    UXt = SP.array(UXt).T
    if UXt.size == 0: #predictors are homogeneous
        return mBest, sBest, left_mean, right_mean, score_best
    else:
        #print UXt
#         print X[noderange]
#         print ''
#         print ''
        # test all transformed predictors
        scores = -NP.ones(cnt)*float('inf')
        UC = SP.dot(U.T,C)
        ########################
        #finding the best split#
        ########################
        score_0 = lmm_fast.nLLeval(ldelta,Uy[:,0],UC,S)
        for snp_cnt in SP.arange(cnt):
            UX=SP.hstack((UXt[:,snp_cnt:snp_cnt+1], UC))
            scores[snp_cnt] = -lmm_fast.nLLeval(ldelta,Uy[:,0],UX,S)
            scores[snp_cnt] += score_0
        ############################
        ###evaluate the new means###
        ############################
        kBest = SP.argmax(scores)
        score_best = scores[kBest]
        sBest = s[kBest]
        if score_best > 0:
                sBest = s[kBest]
                score_best = scores[kBest]
                UX=SP.hstack((UXt[:,kBest:kBest+1], UC))
                _, beta,_ = lmm_fast.nLLeval(ldelta, Uy[:,0], UX, S, MLparams=True)
                mBest = feature_map[kBest]
                CX = SP.zeros_like(Uy)
                CX[noderange] = SP.int_(X[noderange,mBest:mBest+1] > sBest)
                C_new = SP.hstack((CX,C))
                mean = SP.dot(C_new,beta.reshape(beta.size, -1)) #TODO:is this the correct way?
                left_mean = ((mean[noderange])[CX[noderange]==0])[0]
                right_mean = ((mean[noderange])[CX[noderange]==1])[0]
        return mBest, sBest, left_mean, right_mean, score_best

if __name__== '__main__':
    node = 5
    # test building of covariates
    subsample = SP.arange(10,dtype='int')
    start_index =  [0,0,4,4,5]
    end_index = [10,4,10,5,10]
    node_labels = [0,1,2,5,6]
    parents = [0,0,0,2,2]
    node_ind = 4
    print get_covariates(node_ind, node_labels[node_ind], parents, subsample, start_index, end_index)
